#!/usr/bin/env python
#

"""Tests for saml2.md"""

__author__ = "roland.hedberg@umu.se (Roland Hedberg)"

import ds_data
import md_data

import saml2
from saml2 import extension_element_to_element
from saml2 import md
from saml2 import saml
from saml2 import samlp
from saml2 import xmldsig as ds
from saml2.extension import idpdisc
from saml2.extension import shibmd


class TestEndpointType:
    def setup_class(self):
        self.endpoint = md.EndpointType_()

    def testAccessors(self):
        """Test for EndpointType accessors"""
        self.endpoint.binding = saml2.BINDING_HTTP_POST
        self.endpoint.location = "http://www.example.com/endpoint"
        self.endpoint.response_location = "http://www.example.com/response"
        print(self.endpoint.__class__.c_attributes.items())
        new_endpoint = md.endpoint_type__from_string(self.endpoint.to_string())
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"

    def testUsingTestData(self):
        """Test for endpoint_type_from_string() using test data."""
        new_endpoint = md.endpoint_type__from_string(md_data.TEST_ENDPOINT)
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"


class TestIndexedEndpointType:
    def setup_class(self):
        self.i_e = md.IndexedEndpointType_()

    def testAccessors(self):
        """Test for IndexedEndpointType accessors"""
        self.i_e.binding = saml2.BINDING_HTTP_POST
        self.i_e.location = "http://www.example.com/endpoint"
        self.i_e.response_location = "http://www.example.com/response"
        self.i_e.index = "1"
        self.i_e.is_default = "false"
        new_i_e = md.indexed_endpoint_type__from_string(self.i_e.to_string())
        assert new_i_e.binding == saml2.BINDING_HTTP_POST
        assert new_i_e.location == "http://www.example.com/endpoint"
        assert new_i_e.response_location == "http://www.example.com/response"
        assert new_i_e.index == "1"
        assert new_i_e.is_default == "false"

    def testUsingTestData(self):
        """Test for indexed_endpoint_type_from_string() using test data."""
        new_i_e = md.indexed_endpoint_type__from_string(md_data.TEST_INDEXED_ENDPOINT)
        assert new_i_e.binding == saml2.BINDING_HTTP_POST
        assert new_i_e.location == "http://www.example.com/endpoint"
        assert new_i_e.response_location == "http://www.example.com/response"
        assert new_i_e.index == "1"
        assert new_i_e.is_default == "false"


class TestExtensions:
    def setup_class(self):
        self.extensions = md.Extensions()

    def testAccessors(self):
        """Test for Extensions accessors"""
        self.extensions.extension_elements.append(
            saml2.extension_element_from_string(
                """<?xml version='1.0' encoding='UTF-8'?>
                <hoge>fuga</hoge>
                """
            )
        )
        new_extensions = md.extensions_from_string(self.extensions.to_string())
        assert new_extensions.extension_elements[0].tag == "hoge"
        assert new_extensions.extension_elements[0].text.strip() == "fuga"


class TestOrganizationName:
    def setup_class(self):
        self.organization_name = md.OrganizationName()

    def testAccessors(self):
        """Test for OrganizationName accessors"""
        self.organization_name.lang = "en"
        self.organization_name.text = "SIOS Technology, Inc."
        new_organization_name = md.organization_name_from_string(self.organization_name.to_string())
        assert new_organization_name.lang == "en"
        assert new_organization_name.text.strip() == "SIOS Technology, Inc."

    def testUsingTestData(self):
        """Test for organization_name_from_string() using test data."""
        new_organization_name = md.organization_name_from_string(md_data.TEST_ORGANIZATION_NAME)
        print(new_organization_name.keyswv())
        assert new_organization_name.lang == "se"
        assert new_organization_name.text.strip() == "Catalogix"


class TestOrganizationDisplayName:
    def setup_class(self):
        self.od_name = md.OrganizationDisplayName()

    def testAccessors(self):
        """Test for OrganizationDisplayName accessors"""
        self.od_name.lang = "en"
        self.od_name.text = "SIOS"
        new_od_name = md.organization_display_name_from_string(self.od_name.to_string())
        assert new_od_name.lang == "en"
        assert new_od_name.text.strip() == "SIOS"

    def testUsingTestData(self):
        """Test for organization_display_name_from_string() using test data."""
        new_od_name = md.organization_display_name_from_string(md_data.TEST_ORGANIZATION_DISPLAY_NAME)
        assert new_od_name.lang == "se"
        assert new_od_name.text.strip() == "Catalogix"


class TestOrganizationURL:
    def setup_class(self):
        self.organization_url = md.OrganizationURL()

    def testAccessors(self):
        """Test for OrganizationURL accessors"""
        self.organization_url.lang = "ja"
        self.organization_url.text = "http://www.example.com/"
        print(self.organization_url.to_string())
        new_organization_url = md.organization_url_from_string(self.organization_url.to_string())
        assert new_organization_url.lang == "ja"
        assert new_organization_url.text.strip() == "http://www.example.com/"

    def testUsingTestData(self):
        """Test for organization_url_from_string() using test data."""
        new_organization_url = md.organization_url_from_string(md_data.TEST_ORGANIZATION_URL)
        assert new_organization_url.lang == "no"
        assert new_organization_url.text.strip() == "http://www.example.com/"


class TestOrganization:
    def setup_class(self):
        self.organization = md.Organization()

    def testAccessors(self):
        """Test for Organization accessors"""
        self.organization.extensions = md.Extensions()
        self.organization.organization_name.append(md.organization_name_from_string(md_data.TEST_ORGANIZATION_NAME))
        self.organization.organization_display_name.append(
            md.organization_display_name_from_string(md_data.TEST_ORGANIZATION_DISPLAY_NAME)
        )
        self.organization.organization_url.append(md.organization_url_from_string(md_data.TEST_ORGANIZATION_URL))
        new_organization = md.organization_from_string(self.organization.to_string())
        assert isinstance(new_organization.extensions, md.Extensions)
        assert isinstance(new_organization.organization_name[0], md.OrganizationName)
        assert isinstance(new_organization.organization_display_name[0], md.OrganizationDisplayName)
        assert isinstance(new_organization.organization_url[0], md.OrganizationURL)
        assert new_organization.organization_name[0].text.strip() == "Catalogix"
        assert new_organization.organization_name[0].lang == "se"
        assert new_organization.organization_display_name[0].text.strip() == "Catalogix"
        assert new_organization.organization_display_name[0].lang == "se"
        assert new_organization.organization_url[0].text.strip() == "http://www.example.com/"
        assert new_organization.organization_url[0].lang == "no"

    def testUsingTestData(self):
        """Test for organization_from_string() using test data."""
        new_organization = md.organization_from_string(md_data.TEST_ORGANIZATION)
        assert isinstance(new_organization.extensions, md.Extensions)
        assert isinstance(new_organization.organization_name[0], md.OrganizationName)
        assert isinstance(new_organization.organization_display_name[0], md.OrganizationDisplayName)
        assert isinstance(new_organization.organization_url[0], md.OrganizationURL)
        assert new_organization.organization_name[0].text.strip() == "Catalogix AB"
        assert new_organization.organization_name[0].lang == "se"
        assert new_organization.organization_display_name[0].text.strip() == "Catalogix AS"
        assert new_organization.organization_display_name[0].lang == "no"
        assert new_organization.organization_url[0].text.strip() == "http://www.example.com/"
        assert new_organization.organization_url[0].lang == "en"


class TestContactPerson:
    def setup_class(self):
        self.contact_person = md.ContactPerson()

    def testAccessors(self):
        """Test for ContactPerson accessors"""
        self.contact_person.contact_type = "technical"
        self.contact_person.extensions = md.Extensions()
        self.contact_person.company = md.Company(text="SIOS Technology, Inc.")
        self.contact_person.given_name = md.GivenName(text="Takashi")
        self.contact_person.sur_name = md.SurName(text="Matsuo")
        self.contact_person.email_address.append(md.EmailAddress(text="tmatsuo@example.com"))
        self.contact_person.email_address.append(md.EmailAddress(text="tmatsuo@shehas.net"))
        self.contact_person.telephone_number.append(md.TelephoneNumber(text="00-0000-0000"))
        new_contact_person = md.contact_person_from_string(self.contact_person.to_string())
        assert new_contact_person.contact_type == "technical"
        assert isinstance(new_contact_person.extensions, md.Extensions)
        assert new_contact_person.company.text.strip() == "SIOS Technology, " "Inc."
        assert new_contact_person.given_name.text.strip() == "Takashi"
        assert new_contact_person.sur_name.text.strip() == "Matsuo"
        assert new_contact_person.email_address[0].text.strip() == "tmatsuo@example.com"
        assert new_contact_person.email_address[1].text.strip() == "tmatsuo@shehas.net"
        assert new_contact_person.telephone_number[0].text.strip() == "00-0000-0000"

    def testUsingTestData(self):
        """Test for contact_person_from_string() using test data."""
        new_contact_person = md.contact_person_from_string(md_data.TEST_CONTACT_PERSON)
        assert new_contact_person.contact_type == "technical"
        assert isinstance(new_contact_person.extensions, md.Extensions)
        assert new_contact_person.company.text.strip() == "SIOS Technology, " "Inc."
        assert new_contact_person.given_name.text.strip() == "Takashi"
        assert new_contact_person.sur_name.text.strip() == "Matsuo"
        assert new_contact_person.email_address[0].text.strip() == "tmatsuo@example.com"
        assert new_contact_person.email_address[1].text.strip() == "tmatsuo@shehas.net"
        assert new_contact_person.telephone_number[0].text.strip() == "00-0000-0000"


class TestAdditionalMetadataLocation:
    def setup_class(self):
        self.additional_metadata_location = md.AdditionalMetadataLocation()

    def testAccessors(self):
        """Test for AdditionalMetadataLocation accessors"""
        self.additional_metadata_location.namespace = "http://www.example.com/namespace"
        self.additional_metadata_location.text = "http://www.example.com/AdditionalMetadataLocation"
        new_additional_metadata_location = md.additional_metadata_location_from_string(
            self.additional_metadata_location.to_string()
        )
        assert new_additional_metadata_location.namespace == "http://www.example.com/namespace"
        assert new_additional_metadata_location.text.strip() == "http://www.example.com/AdditionalMetadataLocation"

    def testUsingTestData(self):
        """Test for additional_metadata_location_from_string() using test
        data."""
        new_additional_metadata_location = md.additional_metadata_location_from_string(
            md_data.TEST_ADDITIONAL_METADATA_LOCATION
        )
        assert new_additional_metadata_location.namespace == "http://www.example.com/namespace"
        assert new_additional_metadata_location.text.strip() == "http://www.example.com/AdditionalMetadataLocation"


# class TestKeySize:
#
#   def setup_class(self):
#     self.key_size = md.KeySize()
#
#   def testAccessors(self):
#     """Test for KeySize accessors"""
#     self.key_size.text = "128"
#     new_key_size = md.key_size_from_string(self.key_size.to_string())
#     assert new_key_size.text.strip() == "128"
#
#   def testUsingTestData(self):
#     """Test for key_size_from_string() using test data."""
#     new_key_size = md.key_size_from_string(md_data.TEST_KEY_SIZE)
#     assert new_key_size.text.strip() == "128"


# class TestOAEPparams:
#
#   def setup_class(self):
#     self.oaep_params = md.OAEPparams()
#
#   def testAccessors(self):
#     """Test for OAEPparams accessors"""
#     self.oaep_params.text = "9lWu3Q=="
#     new_oaep_params = md.oae_pparams_from_string(self.oaep_params.to_string())
#     assert new_oaep_params.text.strip() == "9lWu3Q=="
#
#   def testUsingTestData(self):
#     """Test for oae_pparams_from_string() using test data."""
#     new_oaep_params = md.oae_pparams_from_string(md_data.TEST_OAEP_PARAMS)
#     assert new_oaep_params.text.strip() == "9lWu3Q=="


class TestEncryptionMethod:
    def setup_class(self):
        self.encryption_method = md.EncryptionMethod()

    def testAccessors(self):
        """Test for EncryptionMethod accessors"""
        self.encryption_method.algorithm = "http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p"
        new_encryption_method = md.encryption_method_from_string(self.encryption_method.to_string())
        assert new_encryption_method.algorithm == "http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p"

    def testUsingTestData(self):
        """Test for encryption_method_from_string() using test data."""
        new_encryption_method = md.encryption_method_from_string(md_data.TEST_ENCRYPTION_METHOD)
        assert new_encryption_method.algorithm == "http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p"
        assert new_encryption_method.oae_pparams.text.strip() == "9lWu3Q=="


class TestKeyDescriptor:
    def setup_class(self):
        self.key_descriptor = md.KeyDescriptor()

    def testAccessors(self):
        """Test for KeyDescriptor accessors"""

        self.key_descriptor.use = "signing"
        self.key_descriptor.key_info = ds.key_info_from_string(ds_data.TEST_KEY_INFO)
        self.key_descriptor.encryption_method.append(md.encryption_method_from_string(md_data.TEST_ENCRYPTION_METHOD))
        new_key_descriptor = md.key_descriptor_from_string(self.key_descriptor.to_string())
        assert new_key_descriptor.use == "signing"
        assert isinstance(new_key_descriptor.key_info, ds.KeyInfo)
        assert isinstance(new_key_descriptor.encryption_method[0], md.EncryptionMethod)

    def testUsingTestData(self):
        """Test for key_descriptor_from_string() using test data."""
        new_key_descriptor = md.key_descriptor_from_string(md_data.TEST_KEY_DESCRIPTOR)
        assert new_key_descriptor.use == "signing"
        assert isinstance(new_key_descriptor.key_info, ds.KeyInfo)
        assert isinstance(new_key_descriptor.encryption_method[0], md.EncryptionMethod)


class TestRoleDescriptor:
    def setup_class(self):
        self.role_descriptor = md.RoleDescriptor()

    def testAccessors(self):
        """Test for RoleDescriptor accessors"""
        self.role_descriptor.id = "ID"
        self.role_descriptor.valid_until = "2008-09-14T01:05:02Z"
        self.role_descriptor.cache_duration = "10:00:00:00"
        self.role_descriptor.protocol_support_enumeration = samlp.NAMESPACE
        self.role_descriptor.error_url = "http://www.example.com/errorURL"
        self.role_descriptor.signature = ds.Signature()
        self.role_descriptor.extensions = md.Extensions()
        self.role_descriptor.key_descriptor.append(md.key_descriptor_from_string(md_data.TEST_KEY_DESCRIPTOR))
        self.role_descriptor.organization = md.Organization()
        self.role_descriptor.contact_person.append(md.ContactPerson())

        new_role_descriptor = md.role_descriptor_from_string(self.role_descriptor.to_string())
        assert new_role_descriptor.id == "ID"
        assert new_role_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_role_descriptor.cache_duration == "10:00:00:00"
        assert new_role_descriptor.protocol_support_enumeration == samlp.NAMESPACE
        assert new_role_descriptor.error_url == "http://www.example.com/errorURL"
        assert isinstance(new_role_descriptor.signature, ds.Signature)
        assert isinstance(new_role_descriptor.extensions, md.Extensions)
        assert isinstance(new_role_descriptor.key_descriptor[0], md.KeyDescriptor)
        assert isinstance(new_role_descriptor.organization, md.Organization)
        assert isinstance(new_role_descriptor.contact_person[0], md.ContactPerson)

    def testUsingTestData(self):
        """Test for role_descriptor_from_string() using test data."""
        new_role_descriptor = md.role_descriptor_from_string(md_data.TEST_ROLE_DESCRIPTOR)
        assert new_role_descriptor.id == "ID"
        assert new_role_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_role_descriptor.cache_duration == "10:00:00:00"
        assert new_role_descriptor.protocol_support_enumeration == samlp.NAMESPACE
        assert new_role_descriptor.error_url == "http://www.example.com/errorURL"
        assert isinstance(new_role_descriptor.signature, ds.Signature)
        assert isinstance(new_role_descriptor.extensions, md.Extensions)
        assert isinstance(new_role_descriptor.key_descriptor[0], md.KeyDescriptor)
        assert isinstance(new_role_descriptor.organization, md.Organization)
        assert isinstance(new_role_descriptor.contact_person[0], md.ContactPerson)


# class TestSSODescriptor:
#   def setup_class(self):
#     self.sso_descriptor = md.SSODescriptorType_()
#
#   def testAccessors(self):
#     """Test for SSODescriptorType accessors"""
#     self.sso_descriptor.id = "ID"
#     self.sso_descriptor.valid_until = "2008-09-14T01:05:02Z"
#     self.sso_descriptor.cache_duration = "10:00:00:00"
#     self.sso_descriptor.protocol_support_enumeration = samlp.NAMESPACE
#     self.sso_descriptor.error_url = "http://www.example.com/errorURL"
#     self.sso_descriptor.signature = ds.Signature()
#     self.sso_descriptor.extensions = md.Extensions()
#     self.sso_descriptor.key_descriptor.append(md.key_descriptor_from_string(
#       md_data.TEST_KEY_DESCRIPTOR))
#     self.sso_descriptor.organization = md.Organization()
#     self.sso_descriptor.contact_person.append(md.ContactPerson())
#     self.sso_descriptor.artifact_resolution_service.append(
#       md.ArtifactResolutionService())
#     self.sso_descriptor.single_logout_service.append(
#       md.SingleLogoutService())
#     self.sso_descriptor.manage_name_id_service.append(
#       md.ManageNameIDService())
#     self.sso_descriptor.name_id_format.append(
#       md.NameIDFormat())
#
#     new_sso_descriptor = md.sso_descriptor_type__from_string(
#       self.sso_descriptor.to_string())
#     assert new_sso_descriptor.id == "ID"
#     assert new_sso_descriptor.valid_until == "2008-09-14T01:05:02Z"
#     assert new_sso_descriptor.cache_duration == "10:00:00:00"
#     assert new_sso_descriptor.protocol_support_enumeration == samlp.NAMESPACE
#     assert new_sso_descriptor.error_url == "http://www.example.com/errorURL"
#     assert isinstance(new_sso_descriptor.signature, ds.Signature)
#     assert isinstance(new_sso_descriptor.extensions, md.Extensions)
#     assert isinstance(new_sso_descriptor.key_descriptor[0],
#                             md.KeyDescriptor)
#     assert isinstance(new_sso_descriptor.organization, md.Organization)
#     assert isinstance(new_sso_descriptor.contact_person[0],
#                             md.ContactPerson)
#     assert isinstance(new_sso_descriptor.artifact_resolution_service[0],
#                             md.ArtifactResolutionService)
#     assert isinstance(new_sso_descriptor.single_logout_service[0],
#                             md.SingleLogoutService)
#     assert isinstance(new_sso_descriptor.manage_name_id_service[0],
#                             md.ManageNameIDService)
#     assert isinstance(new_sso_descriptor.name_id_format[0],
#                             md.NameIDFormat)
#


class TestArtifactResolutionService:
    def setup_class(self):
        self.i_e = md.ArtifactResolutionService()

    def testAccessors(self):
        """Test for ArtifactResolutionService accessors"""
        self.i_e.binding = saml2.BINDING_HTTP_POST
        self.i_e.location = "http://www.example.com/endpoint"
        self.i_e.response_location = "http://www.example.com/response"
        self.i_e.index = "1"
        self.i_e.is_default = "false"
        new_i_e = md.artifact_resolution_service_from_string(self.i_e.to_string())
        assert new_i_e.binding == saml2.BINDING_HTTP_POST
        assert new_i_e.location == "http://www.example.com/endpoint"
        assert new_i_e.response_location == "http://www.example.com/response"
        assert new_i_e.index == "1"
        assert new_i_e.is_default == "false"

    def testUsingTestData(self):
        """Test for artifact_resolution_service_from_string() using test
        data."""
        new_i_e = md.artifact_resolution_service_from_string(md_data.TEST_ARTIFACT_RESOLUTION_SERVICE)
        assert new_i_e.binding == saml2.BINDING_HTTP_POST
        assert new_i_e.location == "http://www.example.com/endpoint"
        assert new_i_e.response_location == "http://www.example.com/response"
        assert new_i_e.index == "1"
        assert new_i_e.is_default == "false"


class TestSingleLogout:
    def setup_class(self):
        self.endpoint = md.SingleLogoutService()

    def testAccessors(self):
        """Test for SingleLogoutService accessors"""
        self.endpoint.binding = saml2.BINDING_HTTP_POST
        self.endpoint.location = "http://www.example.com/endpoint"
        self.endpoint.response_location = "http://www.example.com/response"
        new_endpoint = md.single_logout_service_from_string(self.endpoint.to_string())
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"

    def testUsingTestData(self):
        """Test for single_logout_service_from_string() using test data."""
        new_endpoint = md.single_logout_service_from_string(md_data.TEST_SINGLE_LOGOUT_SERVICE)
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"


class TestManageNameIDService:
    def setup_class(self):
        self.endpoint = md.ManageNameIDService()

    def testAccessors(self):
        """Test for ManageNameIDService accessors"""
        self.endpoint.binding = saml2.BINDING_HTTP_POST
        self.endpoint.location = "http://www.example.com/endpoint"
        self.endpoint.response_location = "http://www.example.com/response"
        new_endpoint = md.manage_name_id_service_from_string(self.endpoint.to_string())
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"

    def testUsingTestData(self):
        """Test for manage_name_id_service_from_string() using test data."""
        new_endpoint = md.manage_name_id_service_from_string(md_data.TEST_MANAGE_NAMEID_SERVICE)
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"


class TestNameIDFormat:
    def setup_class(self):
        self.name_id_format = md.NameIDFormat()

    def testAccessors(self):
        """Test for NameIDFormat accessors"""
        self.name_id_format.text = saml.NAMEID_FORMAT_EMAILADDRESS
        new_name_id_format = md.name_id_format_from_string(self.name_id_format.to_string())
        assert new_name_id_format.text.strip() == saml.NAMEID_FORMAT_EMAILADDRESS

    def testUsingTestData(self):
        """Test for name_id_format_from_string() using test data."""
        new_name_id_format = md.name_id_format_from_string(md_data.TEST_NAME_ID_FORMAT)
        assert new_name_id_format.text.strip() == saml.NAMEID_FORMAT_EMAILADDRESS


class TestSingleSignOnService:
    def setup_class(self):
        self.endpoint = md.SingleSignOnService()

    def testAccessors(self):
        """Test for SingelSignOnService accessors"""
        self.endpoint.binding = saml2.BINDING_HTTP_POST
        self.endpoint.location = "http://www.example.com/endpoint"
        self.endpoint.response_location = "http://www.example.com/response"
        new_endpoint = md.single_sign_on_service_from_string(self.endpoint.to_string())
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"

    def testUsingTestData(self):
        """Test for SingelSignOn_service_from_string() using test data."""
        new_endpoint = md.single_sign_on_service_from_string(md_data.TEST_SINGLE_SIGN_ON_SERVICE)
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"


class TestNameIDMappingService:
    def setup_class(self):
        self.endpoint = md.NameIDMappingService()

    def testAccessors(self):
        """Test for NameIDMappingService accessors"""
        self.endpoint.binding = saml2.BINDING_HTTP_POST
        self.endpoint.location = "http://www.example.com/endpoint"
        self.endpoint.response_location = "http://www.example.com/response"
        new_endpoint = md.name_id_mapping_service_from_string(self.endpoint.to_string())
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"

    def testUsingTestData(self):
        """Test for name_id_mapping_service_from_string() using test data."""
        new_endpoint = md.name_id_mapping_service_from_string(md_data.TEST_NAME_ID_MAPPING_SERVICE)
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"


class TestAssertionIDRequestService:
    def setup_class(self):
        self.endpoint = md.AssertionIDRequestService()

    def testAccessors(self):
        """Test for AssertionIDRequestService accessors"""
        self.endpoint.binding = saml2.BINDING_HTTP_POST
        self.endpoint.location = "http://www.example.com/endpoint"
        self.endpoint.response_location = "http://www.example.com/response"
        new_endpoint = md.assertion_id_request_service_from_string(self.endpoint.to_string())
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"

    def testUsingTestData(self):
        """Test for assertion_id_request_service_from_string() using test
        data."""
        new_endpoint = md.assertion_id_request_service_from_string(md_data.TEST_ASSERTION_ID_REQUEST_SERVICE)
        assert new_endpoint.binding == saml2.BINDING_HTTP_POST
        assert new_endpoint.location == "http://www.example.com/endpoint"
        assert new_endpoint.response_location == "http://www.example.com/response"


class TestAttributeProfile:
    def setup_class(self):
        self.attribute_profile = md.AttributeProfile()

    def testAccessors(self):
        """Test for AttributeProfile accessors"""
        self.attribute_profile.text = saml.PROFILE_ATTRIBUTE_BASIC
        new_attribute_profile = md.attribute_profile_from_string(self.attribute_profile.to_string())
        assert new_attribute_profile.text.strip() == saml.PROFILE_ATTRIBUTE_BASIC

    def testUsingTestData(self):
        """Test for name_id_format_from_string() using test data."""
        new_attribute_profile = md.attribute_profile_from_string(md_data.TEST_ATTRIBUTE_PROFILE)
        assert new_attribute_profile.text.strip() == saml.PROFILE_ATTRIBUTE_BASIC


class TestIDPSSODescriptor:
    def setup_class(self):
        self.idp_sso_descriptor = md.IDPSSODescriptor()

    def testAccessors(self):
        """Test for IDPSSODescriptor accessors"""
        self.idp_sso_descriptor.id = "ID"
        self.idp_sso_descriptor.valid_until = "2008-09-14T01:05:02Z"
        self.idp_sso_descriptor.cache_duration = "10:00:00:00"
        self.idp_sso_descriptor.protocol_support_enumeration = samlp.NAMESPACE
        self.idp_sso_descriptor.error_url = "http://www.example.com/errorURL"
        self.idp_sso_descriptor.signature = ds.Signature()
        self.idp_sso_descriptor.extensions = md.Extensions()
        self.idp_sso_descriptor.key_descriptor.append(md.key_descriptor_from_string(md_data.TEST_KEY_DESCRIPTOR))
        self.idp_sso_descriptor.organization = md.Organization()
        self.idp_sso_descriptor.contact_person.append(md.ContactPerson())
        self.idp_sso_descriptor.artifact_resolution_service.append(md.ArtifactResolutionService())
        self.idp_sso_descriptor.single_logout_service.append(md.SingleLogoutService())
        self.idp_sso_descriptor.manage_name_id_service.append(md.ManageNameIDService())
        self.idp_sso_descriptor.name_id_format.append(md.NameIDFormat())
        self.idp_sso_descriptor.want_authn_requests_signed = "true"
        self.idp_sso_descriptor.single_sign_on_service.append(md.SingleSignOnService())
        self.idp_sso_descriptor.name_id_mapping_service.append(md.NameIDMappingService())
        self.idp_sso_descriptor.assertion_id_request_service.append(md.AssertionIDRequestService())
        self.idp_sso_descriptor.attribute_profile.append(md.AttributeProfile())
        self.idp_sso_descriptor.attribute.append(saml.Attribute())

        new_idp_sso_descriptor = md.idpsso_descriptor_from_string(self.idp_sso_descriptor.to_string())
        assert new_idp_sso_descriptor.id == "ID"
        assert new_idp_sso_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_idp_sso_descriptor.cache_duration == "10:00:00:00"
        assert new_idp_sso_descriptor.protocol_support_enumeration == samlp.NAMESPACE
        assert new_idp_sso_descriptor.error_url == "http://www.example.com/errorURL"
        assert isinstance(new_idp_sso_descriptor.signature, ds.Signature)
        assert isinstance(new_idp_sso_descriptor.extensions, md.Extensions)
        assert isinstance(new_idp_sso_descriptor.key_descriptor[0], md.KeyDescriptor)
        assert isinstance(new_idp_sso_descriptor.organization, md.Organization)
        assert isinstance(new_idp_sso_descriptor.contact_person[0], md.ContactPerson)
        assert isinstance(new_idp_sso_descriptor.artifact_resolution_service[0], md.ArtifactResolutionService)
        assert isinstance(new_idp_sso_descriptor.single_logout_service[0], md.SingleLogoutService)
        assert isinstance(new_idp_sso_descriptor.manage_name_id_service[0], md.ManageNameIDService)
        assert isinstance(new_idp_sso_descriptor.name_id_format[0], md.NameIDFormat)
        assert new_idp_sso_descriptor.want_authn_requests_signed == "true"
        assert isinstance(new_idp_sso_descriptor.single_sign_on_service[0], md.SingleSignOnService)
        assert isinstance(new_idp_sso_descriptor.name_id_mapping_service[0], md.NameIDMappingService)
        assert isinstance(new_idp_sso_descriptor.assertion_id_request_service[0], md.AssertionIDRequestService)
        assert isinstance(new_idp_sso_descriptor.attribute_profile[0], md.AttributeProfile)
        assert isinstance(new_idp_sso_descriptor.attribute[0], saml.Attribute)

    def testUsingTestData(self):
        """Test for idpsso_descriptor_from_string() using test data."""
        new_idp_sso_descriptor = md.idpsso_descriptor_from_string(md_data.TEST_IDP_SSO_DESCRIPTOR)
        assert new_idp_sso_descriptor.id == "ID"
        assert new_idp_sso_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_idp_sso_descriptor.cache_duration == "10:00:00:00"
        assert new_idp_sso_descriptor.protocol_support_enumeration == samlp.NAMESPACE
        assert new_idp_sso_descriptor.error_url == "http://www.example.com/errorURL"
        assert isinstance(new_idp_sso_descriptor.signature, ds.Signature)
        assert isinstance(new_idp_sso_descriptor.extensions, md.Extensions)
        assert isinstance(new_idp_sso_descriptor.key_descriptor[0], md.KeyDescriptor)
        assert isinstance(new_idp_sso_descriptor.organization, md.Organization)
        assert isinstance(new_idp_sso_descriptor.contact_person[0], md.ContactPerson)
        assert isinstance(new_idp_sso_descriptor.artifact_resolution_service[0], md.ArtifactResolutionService)
        assert isinstance(new_idp_sso_descriptor.single_logout_service[0], md.SingleLogoutService)
        assert isinstance(new_idp_sso_descriptor.manage_name_id_service[0], md.ManageNameIDService)
        assert isinstance(new_idp_sso_descriptor.name_id_format[0], md.NameIDFormat)
        assert new_idp_sso_descriptor.want_authn_requests_signed == "true"
        assert isinstance(new_idp_sso_descriptor.single_sign_on_service[0], md.SingleSignOnService)
        assert isinstance(new_idp_sso_descriptor.name_id_mapping_service[0], md.NameIDMappingService)
        assert isinstance(new_idp_sso_descriptor.assertion_id_request_service[0], md.AssertionIDRequestService)
        assert isinstance(new_idp_sso_descriptor.attribute_profile[0], md.AttributeProfile)
        assert isinstance(new_idp_sso_descriptor.attribute[0], saml.Attribute)

    def testUsingScope(self):
        descriptor = md.IDPSSODescriptor()
        scope = shibmd.Scope()
        scope.text = "example.org"
        scope.regexp = "false"
        descriptor.extensions = md.Extensions()
        ext = saml2.element_to_extension_element(scope)
        descriptor.extensions.extension_elements.append(ext)
        exts = descriptor.extensions
        assert len(exts.extension_elements) == 1
        elem = exts.extension_elements[0]
        inst = saml2.extension_element_to_element(elem, shibmd.ELEMENT_FROM_STRING, namespace=shibmd.NAMESPACE)
        assert isinstance(inst, shibmd.Scope)
        assert inst.text == "example.org"
        assert inst.regexp == "false"


class TestAssertionConsumerService:
    def setup_class(self):
        self.i_e = md.AssertionConsumerService()

    def testAccessors(self):
        """Test for AssertionConsumerService accessors"""
        self.i_e.binding = saml2.BINDING_HTTP_POST
        self.i_e.location = "http://www.example.com/endpoint"
        self.i_e.response_location = "http://www.example.com/response"
        self.i_e.index = "1"
        self.i_e.is_default = "false"
        new_i_e = md.assertion_consumer_service_from_string(self.i_e.to_string())
        assert new_i_e.binding == saml2.BINDING_HTTP_POST
        assert new_i_e.location == "http://www.example.com/endpoint"
        assert new_i_e.response_location == "http://www.example.com/response"
        assert new_i_e.index == "1"
        assert new_i_e.is_default == "false"

    def testUsingTestData(self):
        """Test for assertion_consumer_service_from_string() using test data."""
        new_i_e = md.assertion_consumer_service_from_string(md_data.TEST_ASSERTION_CONSUMER_SERVICE)
        assert new_i_e.binding == saml2.BINDING_HTTP_POST
        assert new_i_e.location == "http://www.example.com/endpoint"
        assert new_i_e.response_location == "http://www.example.com/response"
        assert new_i_e.index == "1"
        assert new_i_e.is_default == "false"


class TestRequestedAttribute:
    def setup_class(self):
        self.requested_attribute = md.RequestedAttribute()

    def testAccessors(self):
        """Test for RequestedAttribute accessors"""
        assert isinstance(self.requested_attribute, saml.AttributeType_)
        assert isinstance(self.requested_attribute, md.RequestedAttribute)
        assert self.requested_attribute.is_required is None
        self.requested_attribute.is_required = "true"
        new_requested_attribute = md.requested_attribute_from_string(self.requested_attribute.to_string())
        assert new_requested_attribute.is_required == "true"
        assert isinstance(new_requested_attribute, saml.AttributeType_)
        assert isinstance(new_requested_attribute, md.RequestedAttribute)

    def testUsingTestData(self):
        """Test for requested_attribute_from_string() using test data."""
        new_requested_attribute = md.requested_attribute_from_string(md_data.TEST_REQUESTED_ATTRIBUTE)
        assert new_requested_attribute.is_required == "true"
        assert isinstance(new_requested_attribute, saml.AttributeType_)
        assert isinstance(new_requested_attribute, md.RequestedAttribute)


class TestServiceName:
    def setup_class(self):
        self.service_name = md.ServiceName()

    def testAccessors(self):
        """Test for ServiceName accessors"""
        self.service_name.lang = "en"
        self.service_name.text = "SIOS mail"
        new_service_name = md.service_name_from_string(self.service_name.to_string())
        assert new_service_name.lang == "en"
        assert new_service_name.text.strip() == "SIOS mail"

    def testUsingTestData(self):
        """Test for organization_name_from_string() using test data."""
        new_service_name = md.service_name_from_string(md_data.TEST_SERVICE_NAME)
        assert new_service_name.lang == "en"
        assert new_service_name.text.strip() == "Catalogix Whois"


class TestServiceDescription:
    def setup_class(self):
        self.service_description = md.ServiceDescription()

    def testAccessors(self):
        """Test for ServiceDescription accessors"""
        self.service_description.lang = "en"
        self.service_description.text = "SIOS mail service"
        new_service_description = md.service_description_from_string(self.service_description.to_string())
        assert new_service_description.lang == "en"
        assert new_service_description.text.strip() == "SIOS mail service"

    def testUsingTestData(self):
        """Test for organization_name_from_string() using test data."""
        new_service_description = md.service_description_from_string(md_data.TEST_SERVICE_DESCRIPTION)
        assert new_service_description.lang == "en"
        assert new_service_description.text.strip() == "Catalogix Whois Service"


class TestAttributeConsumingService:
    def setup_class(self):
        self.attribute_consuming_service = md.AttributeConsumingService()

    def testAccessors(self):
        """Test for AttributeConsumingService accessors"""
        self.attribute_consuming_service.service_name.append(md.ServiceName())
        self.attribute_consuming_service.service_description.append(md.ServiceDescription())
        self.attribute_consuming_service.requested_attribute.append(md.RequestedAttribute())
        self.attribute_consuming_service.index = "1"
        self.attribute_consuming_service.is_default = "true"

        new_attribute_consuming_service = md.attribute_consuming_service_from_string(
            self.attribute_consuming_service.to_string()
        )
        assert new_attribute_consuming_service.index == "1"
        assert new_attribute_consuming_service.is_default == "true"
        assert isinstance(new_attribute_consuming_service.service_name[0], md.ServiceName)
        assert isinstance(new_attribute_consuming_service.service_description[0], md.ServiceDescription)
        assert isinstance(new_attribute_consuming_service.requested_attribute[0], md.RequestedAttribute)

    def testUsingTestData(self):
        """Test for attribute_consuming_service_from_string() using test
        data."""
        new_attribute_consuming_service = md.attribute_consuming_service_from_string(
            md_data.TEST_ATTRIBUTE_CONSUMING_SERVICE
        )
        assert new_attribute_consuming_service.index == "1"
        assert new_attribute_consuming_service.is_default == "true"
        assert isinstance(new_attribute_consuming_service.service_name[0], md.ServiceName)
        assert isinstance(new_attribute_consuming_service.service_description[0], md.ServiceDescription)
        assert isinstance(new_attribute_consuming_service.requested_attribute[0], md.RequestedAttribute)


class TestSPSSODescriptor:
    def setup_class(self):
        self.sp_sso_descriptor = md.SPSSODescriptor()

    def testAccessors(self):
        """Test for SPSSODescriptor accessors"""
        self.sp_sso_descriptor.id = "ID"
        self.sp_sso_descriptor.valid_until = "2008-09-14T01:05:02Z"
        self.sp_sso_descriptor.cache_duration = "10:00:00:00"
        self.sp_sso_descriptor.protocol_support_enumeration = samlp.NAMESPACE
        self.sp_sso_descriptor.error_url = "http://www.example.com/errorURL"
        self.sp_sso_descriptor.signature = ds.Signature()
        self.sp_sso_descriptor.extensions = md.Extensions()
        self.sp_sso_descriptor.key_descriptor.append(md.key_descriptor_from_string(md_data.TEST_KEY_DESCRIPTOR))
        self.sp_sso_descriptor.organization = md.Organization()
        self.sp_sso_descriptor.contact_person.append(md.ContactPerson())
        self.sp_sso_descriptor.artifact_resolution_service.append(md.ArtifactResolutionService())
        self.sp_sso_descriptor.single_logout_service.append(md.SingleLogoutService())
        self.sp_sso_descriptor.manage_name_id_service.append(md.ManageNameIDService())
        self.sp_sso_descriptor.name_id_format.append(md.NameIDFormat())
        self.sp_sso_descriptor.authn_requests_signed = "true"
        self.sp_sso_descriptor.want_assertions_signed = "true"
        self.sp_sso_descriptor.assertion_consumer_service.append(md.AssertionConsumerService())
        self.sp_sso_descriptor.attribute_consuming_service.append(md.AttributeConsumingService())

        print(self.sp_sso_descriptor)
        new_sp_sso_descriptor = md.spsso_descriptor_from_string(self.sp_sso_descriptor.to_string())
        print(new_sp_sso_descriptor)
        assert new_sp_sso_descriptor.id == "ID"
        assert new_sp_sso_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_sp_sso_descriptor.cache_duration == "10:00:00:00"
        assert new_sp_sso_descriptor.protocol_support_enumeration == samlp.NAMESPACE
        assert new_sp_sso_descriptor.error_url == "http://www.example.com/errorURL"
        assert isinstance(new_sp_sso_descriptor.signature, ds.Signature)
        assert isinstance(new_sp_sso_descriptor.extensions, md.Extensions)
        assert isinstance(new_sp_sso_descriptor.key_descriptor[0], md.KeyDescriptor)
        assert isinstance(new_sp_sso_descriptor.organization, md.Organization)
        assert isinstance(new_sp_sso_descriptor.contact_person[0], md.ContactPerson)
        assert isinstance(new_sp_sso_descriptor.artifact_resolution_service[0], md.ArtifactResolutionService)
        assert isinstance(new_sp_sso_descriptor.single_logout_service[0], md.SingleLogoutService)
        assert isinstance(new_sp_sso_descriptor.manage_name_id_service[0], md.ManageNameIDService)
        assert isinstance(new_sp_sso_descriptor.name_id_format[0], md.NameIDFormat)
        assert new_sp_sso_descriptor.authn_requests_signed == "true"
        assert new_sp_sso_descriptor.want_assertions_signed == "true"
        assert isinstance(new_sp_sso_descriptor.assertion_consumer_service[0], md.AssertionConsumerService)
        assert isinstance(new_sp_sso_descriptor.attribute_consuming_service[0], md.AttributeConsumingService)

    def testUsingTestData(self):
        """Test for spsso_descriptor_from_string() using test data."""
        new_sp_sso_descriptor = md.spsso_descriptor_from_string(md_data.TEST_SP_SSO_DESCRIPTOR)
        assert new_sp_sso_descriptor.id == "ID"
        assert new_sp_sso_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_sp_sso_descriptor.cache_duration == "10:00:00:00"
        assert new_sp_sso_descriptor.protocol_support_enumeration == samlp.NAMESPACE
        assert new_sp_sso_descriptor.error_url == "http://www.example.com/errorURL"
        assert isinstance(new_sp_sso_descriptor.signature, ds.Signature)
        assert isinstance(new_sp_sso_descriptor.extensions, md.Extensions)
        print(new_sp_sso_descriptor.extensions.__dict__)
        assert len(new_sp_sso_descriptor.extensions.extension_elements) == 2
        for eelem in new_sp_sso_descriptor.extensions.extension_elements:
            print("EE", eelem.__dict__)
            dp = extension_element_to_element(eelem, idpdisc.ELEMENT_FROM_STRING, idpdisc.NAMESPACE)
            print("DP", dp.c_tag, dp.c_namespace, dp.__dict__)
            assert isinstance(dp, idpdisc.DiscoveryResponse)
        assert isinstance(new_sp_sso_descriptor.key_descriptor[0], md.KeyDescriptor)
        assert isinstance(new_sp_sso_descriptor.organization, md.Organization)
        assert isinstance(new_sp_sso_descriptor.contact_person[0], md.ContactPerson)
        assert isinstance(new_sp_sso_descriptor.artifact_resolution_service[0], md.ArtifactResolutionService)
        assert isinstance(new_sp_sso_descriptor.single_logout_service[0], md.SingleLogoutService)
        assert isinstance(new_sp_sso_descriptor.manage_name_id_service[0], md.ManageNameIDService)
        assert isinstance(new_sp_sso_descriptor.name_id_format[0], md.NameIDFormat)
        assert new_sp_sso_descriptor.authn_requests_signed == "true"
        assert new_sp_sso_descriptor.want_assertions_signed == "true"
        assert isinstance(new_sp_sso_descriptor.assertion_consumer_service[0], md.AssertionConsumerService)
        assert isinstance(new_sp_sso_descriptor.attribute_consuming_service[0], md.AttributeConsumingService)


class TestEntityDescriptor:
    def setup_class(self):
        self.entity_descriptor = md.EntityDescriptor()

    def testAccessors(self):
        """Test for RoleDescriptor accessors"""
        self.entity_descriptor.id = "ID"
        self.entity_descriptor.entity_id = "entityID"
        self.entity_descriptor.valid_until = "2008-09-14T01:05:02Z"
        self.entity_descriptor.cache_duration = "10:00:00:00"

        self.entity_descriptor.signature = ds.Signature()
        self.entity_descriptor.extensions = md.Extensions()
        self.entity_descriptor.role_descriptor.append(md.RoleDescriptor())
        self.entity_descriptor.idpsso_descriptor.append(md.IDPSSODescriptor())
        self.entity_descriptor.spsso_descriptor.append(md.SPSSODescriptor())
        self.entity_descriptor.organization = md.Organization()
        self.entity_descriptor.contact_person.append(md.ContactPerson())
        self.entity_descriptor.additional_metadata_location.append(md.AdditionalMetadataLocation())

        new_entity_descriptor = md.entity_descriptor_from_string(self.entity_descriptor.to_string())
        assert new_entity_descriptor.id == "ID"
        assert new_entity_descriptor.entity_id == "entityID"
        assert new_entity_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_entity_descriptor.cache_duration == "10:00:00:00"
        assert isinstance(new_entity_descriptor.signature, ds.Signature)
        assert isinstance(new_entity_descriptor.extensions, md.Extensions)
        assert isinstance(new_entity_descriptor.role_descriptor[0], md.RoleDescriptor)
        assert isinstance(new_entity_descriptor.idpsso_descriptor[0], md.IDPSSODescriptor)
        assert isinstance(new_entity_descriptor.spsso_descriptor[0], md.SPSSODescriptor)
        assert isinstance(new_entity_descriptor.organization, md.Organization)
        assert isinstance(new_entity_descriptor.contact_person[0], md.ContactPerson)
        assert isinstance(new_entity_descriptor.additional_metadata_location[0], md.AdditionalMetadataLocation)

    def testUsingTestData(self):
        """Test for entity_descriptor_from_string() using test data."""
        new_entity_descriptor = md.entity_descriptor_from_string(md_data.TEST_ENTITY_DESCRIPTOR)
        assert new_entity_descriptor.id == "ID"
        assert new_entity_descriptor.entity_id == "entityID"
        assert new_entity_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_entity_descriptor.cache_duration == "10:00:00:00"
        assert isinstance(new_entity_descriptor.signature, ds.Signature)
        assert isinstance(new_entity_descriptor.extensions, md.Extensions)
        assert isinstance(new_entity_descriptor.role_descriptor[0], md.RoleDescriptor)
        assert isinstance(new_entity_descriptor.idpsso_descriptor[0], md.IDPSSODescriptor)
        assert isinstance(new_entity_descriptor.spsso_descriptor[0], md.SPSSODescriptor)
        assert isinstance(new_entity_descriptor.organization, md.Organization)
        assert isinstance(new_entity_descriptor.contact_person[0], md.ContactPerson)
        assert isinstance(new_entity_descriptor.additional_metadata_location[0], md.AdditionalMetadataLocation)


class TestEntitiesDescriptor:
    def setup_class(self):
        self.entities_descriptor = md.EntitiesDescriptor()

    def testAccessors(self):
        """Test for EntitiesDescriptor accessors"""
        self.entities_descriptor.id = "ID"
        self.entities_descriptor.name = "name"
        self.entities_descriptor.valid_until = "2008-09-14T01:05:02Z"
        self.entities_descriptor.cache_duration = "10:00:00:00"

        self.entities_descriptor.signature = ds.Signature()
        self.entities_descriptor.extensions = md.Extensions()
        self.entities_descriptor.entity_descriptor.append(md.EntityDescriptor())
        self.entities_descriptor.entities_descriptor.append(md.EntitiesDescriptor())

        new_entities_descriptor = md.entities_descriptor_from_string(self.entities_descriptor.to_string())
        assert new_entities_descriptor.id == "ID"
        assert new_entities_descriptor.name == "name"
        assert new_entities_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_entities_descriptor.cache_duration == "10:00:00:00"
        assert isinstance(new_entities_descriptor.signature, ds.Signature)
        assert isinstance(new_entities_descriptor.extensions, md.Extensions)
        assert isinstance(new_entities_descriptor.entity_descriptor[0], md.EntityDescriptor)
        assert isinstance(new_entities_descriptor.entities_descriptor[0], md.EntitiesDescriptor)

    def testUsingTestData(self):
        """Test for entities_descriptor_from_string() using test data."""
        new_entities_descriptor = md.entities_descriptor_from_string(md_data.TEST_ENTITIES_DESCRIPTOR)
        assert new_entities_descriptor.id == "ID"
        assert new_entities_descriptor.name == "name"
        assert new_entities_descriptor.valid_until == "2008-09-14T01:05:02Z"
        assert new_entities_descriptor.cache_duration == "10:00:00:00"
        assert isinstance(new_entities_descriptor.signature, ds.Signature)
        assert isinstance(new_entities_descriptor.extensions, md.Extensions)
        assert isinstance(new_entities_descriptor.entity_descriptor[0], md.EntityDescriptor)
        assert isinstance(new_entities_descriptor.entities_descriptor[0], md.EntitiesDescriptor)


if __name__ == "__main__":
    c = TestIDPSSODescriptor()
    c.setup_class()
    c.testAccessors()



# See http://technet.microsoft.com/en-us/library/cc733065(v=ws.10).aspx
# and http://technet.microsoft.com/en-us/library/ee913589(v=ws.10).aspx
# for information regarding the default claim types supported by
# Microsoft ADFS v1.x.

MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:unspecified",
    "fro": {
        "http://schemas.xmlsoap.org/claims/commonname": "commonName",
        "http://schemas.xmlsoap.org/claims/emailaddress": "emailAddress",
        "http://schemas.xmlsoap.org/claims/group": "group",
        "http://schemas.xmlsoap.org/claims/upn": "upn",
    },
    "to": {
        "commonName": "http://schemas.xmlsoap.org/claims/commonname",
        "emailAddress": "http://schemas.xmlsoap.org/claims/emailaddress",
        "group": "http://schemas.xmlsoap.org/claims/group",
        "upn": "http://schemas.xmlsoap.org/claims/upn",
    },
}



#!/usr/bin/env python
from saml2.saml import NAME_FORMAT_URI


__author__ = "rolandh"

import json


BASE = "http://localhost:8088"

metadata = open("idp_test/idp.xml").read()

info = {
    "entity_id": f"{BASE}/idp.xml",
    "interaction": [
        {
            "matches": {"url": f"{BASE}/login", "title": "IDP test login"},
            "page-type": "login",
            "control": {"type": "form", "set": {"login": "roland", "password": "dianakra"}},
        },
        {
            "matches": {"url": f"{BASE}/sso/redirect", "title": "SAML 2.0 POST"},
            "page-type": "other",
            "control": {"index": 0, "type": "form", "set": {}},
        },
        {
            "matches": {"url": f"{BASE}/sso/post", "title": "SAML 2.0 POST"},
            "page-type": "other",
            "control": {"index": 0, "type": "form", "set": {}},
        },
        {
            "matches": {"url": f"{BASE}/slo/post", "title": "SAML 2.0 POST"},
            "page-type": "other",
            "control": {"index": 0, "type": "form", "set": {}},
        },
    ],
    "metadata": metadata,
    "name_format": NAME_FORMAT_URI,
}

print(json.dumps(info))



#!/usr/bin/env python
import argparse
import logging
import os
import re

from Cookie import SimpleCookie
import sp_conf
from urlparse import parse_qs

from saml2 import BINDING_HTTP_REDIRECT
from saml2 import time_util
from saml2.httputil import NotFound
from saml2.httputil import Redirect
from saml2.httputil import Response
from saml2.httputil import Unauthorized

# from saml2.httputil import ServiceError
from saml2.metadata import create_metadata_string


logger = logging.getLogger("saml2.SP")

args = None
# -----------------------------------------------------------------------------


def dict_to_table(ava, lev=0, width=1):
    txt = [f'<table border={width} bordercolor="black">\n']
    for prop, valarr in ava.items():
        txt.append("<tr>\n")
        if isinstance(valarr, basestring):
            txt.append(f"<th>{str(prop)}</th>\n")
            try:
                txt.append(f"<td>{valarr.encode('utf8')}</td>\n")
            except AttributeError:
                txt.append(f"<td>{valarr}</td>\n")
        elif isinstance(valarr, list):
            i = 0
            n = len(valarr)
            for val in valarr:
                if not i:
                    txt.append(f"<th rowspan={len(valarr)}>{prop}</td>\n")
                else:
                    txt.append("<tr>\n")
                if isinstance(val, dict):
                    txt.append("<td>\n")
                    txt.extend(dict_to_table(val, lev + 1, width - 1))
                    txt.append("</td>\n")
                else:
                    try:
                        txt.append(f"<td>{val.encode('utf8')}</td>\n")
                    except AttributeError:
                        txt.append(f"<td>{val}</td>\n")
                if n > 1:
                    txt.append("</tr>\n")
                n -= 1
                i += 1
        elif isinstance(valarr, dict):
            txt.append(f"<th>{prop}</th>\n")
            txt.append("<td>\n")
            txt.extend(dict_to_table(valarr, lev + 1, width - 1))
            txt.append("</td>\n")
        txt.append("</tr>\n")
    txt.append("</table>\n")
    return txt


def _expiration(timeout, tformat=None):
    if timeout == "now":
        return time_util.instant(tformat)
    else:
        # validity time should match lifetime of assertions
        return time_util.in_a_while(minutes=timeout, format=tformat)


def delete_cookie(environ, name):
    kaka = environ.get("HTTP_COOKIE", "")
    if kaka:
        cookie_obj = SimpleCookie(kaka)
        morsel = cookie_obj.get(name, None)
        cookie = SimpleCookie()
        cookie[name] = morsel
        cookie[name]["expires"] = _expiration("now", "%a, %d-%b-%Y %H:%M:%S CET")
        return tuple(cookie.output().split(": ", 1))
    return None


# ----------------------------------------------------------------------------


# noinspection PyUnusedLocal
def whoami(environ, start_response, user):
    nameid = environ["repoze.who.identity"]["login"]
    ava = environ["repoze.who.identity"]["user"]
    if not nameid:
        return not_authn(environ, start_response)
    if ava:
        response = ["<h2>Your identity is supposed to be</h2>"]
        response.extend(dict_to_table(ava))
    else:
        response = ["<h2>The system did not return any information about you</h2>"]

    response.extend("<a href='logout'>Logout</a>")
    resp = Response(response)
    return resp(environ, start_response)


# noinspection PyUnusedLocal
def not_found(environ, start_response):
    """Called if no URL matches."""
    resp = NotFound("Not Found")
    return resp(environ, start_response)


# noinspection PyUnusedLocal
def not_authn(environ, start_response):
    resp = Unauthorized("Unknown user")
    return resp(environ, start_response)


# noinspection PyUnusedLocal
def slo(environ, start_response, user):
    # so here I might get either a LogoutResponse or a LogoutRequest
    client = environ["repoze.who.plugins"]["saml2auth"]
    sc = client.saml_client

    if "QUERY_STRING" in environ:
        query = parse_qs(environ["QUERY_STRING"])
        logger.info("query: %s", query)
        try:
            response = sc.parse_logout_request_response(query["SAMLResponse"][0], binding=BINDING_HTTP_REDIRECT)
            if response:
                logger.info("LOGOUT response parsed OK")
        except KeyError:
            # return error reply
            response = None

        if response is None:
            request = sc.lo

    headers = []
    delco = delete_cookie(environ, "pysaml2")
    if delco:
        headers.append(delco)
    resp = Redirect("/done", headers=headers)
    return resp(environ, start_response)


# noinspection PyUnusedLocal
def logout(environ, start_response, user):
    # This is where it starts when a user wants to log out
    client = environ["repoze.who.plugins"]["saml2auth"]
    subject_id = environ["repoze.who.identity"]["repoze.who.userid"]
    logger.info("[logout] subject_id: '%s'", subject_id)
    target = "/done"

    # What if more than one
    _dict = client.saml_client.global_logout(subject_id)
    logger.info("[logout] global_logout > %s", _dict)
    rem = environ["repoze.who.plugins"][client.rememberer_name]
    rem.forget(environ, subject_id)

    for key, item in _dict.items():
        if isinstance(item, tuple):
            binding, htargs = item
        else:  # result from logout, should be OK
            pass

    resp = Redirect("Successful Logout", headers=[("Location", target)])
    return resp(environ, start_response)
    # else:
    #     resp = ServiceError("Failed to logout from identity services")
    #     start_response("500 Internal Server Error")
    #     return []


# noinspection PyUnusedLocal
def done(environ, start_response, user):
    # remove cookie and stored info
    logger.info("[done] environ: %s", environ)
    subject_id = environ["repoze.who.identity"]["repoze.who.userid"]
    client = environ["repoze.who.plugins"]["saml2auth"]
    logger.info("[logout done] remaining subjects: %s", client.saml_client.users.subjects())

    start_response("200 OK", [("Content-Type", "text/html")])
    return ["<h3>You are now logged out from this service</h3>"]


# ----------------------------------------------------------------------------

# map urls to functions
urls = [
    (r"whoami$", whoami),
    (r"logout$", logout),
    (r"done$", done),
    (r"slo$", slo),
    (r"^$", whoami),
]

# ----------------------------------------------------------------------------


def metadata(environ, start_response):
    try:
        path = args.path
        if path is None or len(path) == 0:
            path = os.path.dirname(os.path.abspath(__file__))
        if path[-1] != "/":
            path += "/"
        metadata = create_metadata_string(
            f"{path}sp_conf.py", None, args.valid, args.cert, args.keyfile, args.id, args.name, args.sign
        )
        start_response("200 OK", [("Content-Type", "text/xml")])
        return metadata
    except Exception as ex:
        logger.error("An error occured while creating metadata: %s", ex.message)
        return not_found(environ, start_response)


def application(environ, start_response):
    """
    The main WSGI application. Dispatch the current request to
    the functions from above and store the regular expression
    captures in the WSGI environment as `myapp.url_args` so that
    the functions from above can access the url placeholders.

    If nothing matches, call the `not_found` function.

    :param environ: The HTTP application environment
    :param start_response: The application to run when the handling of the
        request is done
    :return: The response as a list of lines
    """
    path = environ.get("PATH_INFO", "").lstrip("/")
    logger.info("<application> PATH: %s", path)

    if path == "metadata":
        return metadata(environ, start_response)

    user = environ.get("REMOTE_USER", "")
    if not user:
        user = environ.get("repoze.who.identity", "")
        logger.info("repoze.who.identity: '%s'", user)
    else:
        logger.info("REMOTE_USER: '%s'", user)
    # logger.info(logging.Logger.manager.loggerDict)
    for regex, callback in urls:
        if user:
            match = re.search(regex, path)
            if match is not None:
                try:
                    environ["myapp.url_args"] = match.groups()[0]
                except IndexError:
                    environ["myapp.url_args"] = path
                return callback(environ, start_response, user)
        else:
            return not_authn(environ, start_response)

    return not_found(environ, start_response)


# ----------------------------------------------------------------------------

from repoze.who.config import make_middleware_with_config


app_with_auth = make_middleware_with_config(application, {"here": "."}, "./who.ini", log_file="repoze_who.log")

# ----------------------------------------------------------------------------
HOST = sp_conf.HOST
PORT = sp_conf.PORT

# allow uwsgi or gunicorn mount
# by moving some initialization out of __name__ == '__main__' section.
# uwsgi -s 0.0.0.0:8087 --protocol http --callable app_with_auth --module idp

if __name__ == "__main__":
    # make_metadata arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", dest="path", help="Path to configuration file.")
    parser.add_argument(
        "-v", dest="valid", default="4", help="How long, in days, the metadata is valid from the time of creation"
    )
    parser.add_argument("-c", dest="cert", help="certificate")
    parser.add_argument("-i", dest="id", help="The ID of the entities descriptor in the metadata")
    parser.add_argument("-k", dest="keyfile", help="A file with a key to sign the metadata with")
    parser.add_argument("-n", dest="name")
    parser.add_argument("-s", dest="sign", action="store_true", help="sign the metadata")
    args = parser.parse_args()

    from wsgiref.simple_server import make_server

    srv = make_server(HOST, PORT, app_with_auth)
    print(f"SP listening on {HOST}:{PORT}")
    srv.serve_forever()



#!/usr/bin/env python

#
# Generated Mon May  2 14:23:33 2011 by parse_xsd.py version 0.4.
#

import saml2
from saml2 import SamlBase
from saml2 import xmldsig as ds


NAMESPACE = "http://www.w3.org/2001/04/xmlenc#"


class KeySizeType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:KeySizeType element"""

    c_tag = "KeySizeType"
    c_namespace = NAMESPACE
    c_value_type = {"base": "integer"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def key_size_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeySizeType_, xml_string)


class CipherValue(SamlBase):

    c_tag = "CipherValue"
    c_namespace = NAMESPACE
    c_value_type = {"base": "base64Binary"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def cipher_value_from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherValue, xml_string)


class TransformsType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:TransformsType element"""

    c_tag = "TransformsType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2000/09/xmldsig#}Transform"] = ("transform", [ds.Transform])
    c_cardinality["transform"] = {"min": 1}
    c_child_order.extend(["transform"])

    def __init__(
        self,
        transform=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.transform = transform or []


def transforms_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(TransformsType_, xml_string)


class KA_Nonce(SamlBase):

    c_tag = "KA_Nonce"
    c_namespace = NAMESPACE
    c_value_type = {"base": "base64Binary"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def k_a__nonce_from_string(xml_string):
    return saml2.create_class_from_xml_string(KA_Nonce, xml_string)


class OriginatorKeyInfo(ds.KeyInfo):

    c_tag = "OriginatorKeyInfo"
    c_namespace = NAMESPACE
    c_children = ds.KeyInfo.c_children.copy()
    c_attributes = ds.KeyInfo.c_attributes.copy()
    c_child_order = ds.KeyInfo.c_child_order[:]
    c_cardinality = ds.KeyInfo.c_cardinality.copy()


def originator_key_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(OriginatorKeyInfo, xml_string)


class RecipientKeyInfo(ds.KeyInfo):

    c_tag = "RecipientKeyInfo"
    c_namespace = NAMESPACE
    c_children = ds.KeyInfo.c_children.copy()
    c_attributes = ds.KeyInfo.c_attributes.copy()
    c_child_order = ds.KeyInfo.c_child_order[:]
    c_cardinality = ds.KeyInfo.c_cardinality.copy()


def recipient_key_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(RecipientKeyInfo, xml_string)


class AgreementMethodType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:AgreementMethodType element"""

    c_tag = "AgreementMethodType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}KA_Nonce"] = ("k_a__nonce", KA_Nonce)
    c_cardinality["k_a__nonce"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2001/04/xmlenc#}OriginatorKeyInfo"] = ("originator_key_info", OriginatorKeyInfo)
    c_cardinality["originator_key_info"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2001/04/xmlenc#}RecipientKeyInfo"] = ("recipient_key_info", RecipientKeyInfo)
    c_cardinality["recipient_key_info"] = {"min": 0, "max": 1}
    c_attributes["Algorithm"] = ("algorithm", "anyURI", True)
    c_child_order.extend(["k_a__nonce", "originator_key_info", "recipient_key_info"])

    def __init__(
        self,
        k_a__nonce=None,
        originator_key_info=None,
        recipient_key_info=None,
        algorithm=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.k_a__nonce = k_a__nonce
        self.originator_key_info = originator_key_info
        self.recipient_key_info = recipient_key_info
        self.algorithm = algorithm


def agreement_method_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AgreementMethodType_, xml_string)


class ReferenceType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:ReferenceType element"""

    c_tag = "ReferenceType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["URI"] = ("uri", "anyURI", True)

    def __init__(
        self,
        uri=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.uri = uri


def reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ReferenceType_, xml_string)


class EncryptionPropertyType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionPropertyType element"""

    c_tag = "EncryptionPropertyType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["Target"] = ("target", "anyURI", False)
    c_attributes["Id"] = ("id", "ID", False)

    def __init__(
        self,
        target=None,
        id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.target = target
        self.id = id


def encryption_property_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionPropertyType_, xml_string)


class KeySize(KeySizeType_):

    c_tag = "KeySize"
    c_namespace = NAMESPACE
    c_children = KeySizeType_.c_children.copy()
    c_attributes = KeySizeType_.c_attributes.copy()
    c_child_order = KeySizeType_.c_child_order[:]
    c_cardinality = KeySizeType_.c_cardinality.copy()


def key_size_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeySize, xml_string)


class OAEPparams(SamlBase):

    c_tag = "OAEPparams"
    c_namespace = NAMESPACE
    c_value_type = {"base": "base64Binary"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def oae_pparams_from_string(xml_string):
    return saml2.create_class_from_xml_string(OAEPparams, xml_string)


class EncryptionMethodType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionMethodType element"""

    c_tag = "EncryptionMethodType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}KeySize"] = ("key_size", KeySize)
    c_cardinality["key_size"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2001/04/xmlenc#}OAEPparams"] = ("oae_pparams", OAEPparams)
    c_cardinality["oae_pparams"] = {"min": 0, "max": 1}
    c_attributes["Algorithm"] = ("algorithm", "anyURI", True)
    c_child_order.extend(["key_size", "oae_pparams"])

    def __init__(
        self,
        key_size=None,
        oae_pparams=None,
        algorithm=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.key_size = key_size
        self.oae_pparams = oae_pparams
        self.algorithm = algorithm


def encryption_method_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionMethodType_, xml_string)


class Transforms(TransformsType_):

    c_tag = "Transforms"
    c_namespace = NAMESPACE
    c_children = TransformsType_.c_children.copy()
    c_attributes = TransformsType_.c_attributes.copy()
    c_child_order = TransformsType_.c_child_order[:]
    c_cardinality = TransformsType_.c_cardinality.copy()


def transforms_from_string(xml_string):
    return saml2.create_class_from_xml_string(Transforms, xml_string)


class CipherReferenceType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:CipherReferenceType element"""

    c_tag = "CipherReferenceType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}Transforms"] = ("transforms", Transforms)
    c_cardinality["transforms"] = {"min": 0, "max": 1}
    c_attributes["URI"] = ("uri", "anyURI", True)
    c_child_order.extend(["transforms"])

    def __init__(
        self,
        transforms=None,
        uri=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.transforms = transforms
        self.uri = uri


def cipher_reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherReferenceType_, xml_string)


class EncryptionMethod(EncryptionMethodType_):

    c_tag = "EncryptionMethod"
    c_namespace = NAMESPACE
    c_children = EncryptionMethodType_.c_children.copy()
    c_attributes = EncryptionMethodType_.c_attributes.copy()
    c_child_order = EncryptionMethodType_.c_child_order[:]
    c_cardinality = EncryptionMethodType_.c_cardinality.copy()


def encryption_method_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionMethod, xml_string)


class AgreementMethod(AgreementMethodType_):
    """The http://www.w3.org/2001/04/xmlenc#:AgreementMethod element"""

    c_tag = "AgreementMethod"
    c_namespace = NAMESPACE
    c_children = AgreementMethodType_.c_children.copy()
    c_attributes = AgreementMethodType_.c_attributes.copy()
    c_child_order = AgreementMethodType_.c_child_order[:]
    c_cardinality = AgreementMethodType_.c_cardinality.copy()


def agreement_method_from_string(xml_string):
    return saml2.create_class_from_xml_string(AgreementMethod, xml_string)


class DataReference(ReferenceType_):

    c_tag = "DataReference"
    c_namespace = NAMESPACE
    c_children = ReferenceType_.c_children.copy()
    c_attributes = ReferenceType_.c_attributes.copy()
    c_child_order = ReferenceType_.c_child_order[:]
    c_cardinality = ReferenceType_.c_cardinality.copy()


def data_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(DataReference, xml_string)


class KeyReference(ReferenceType_):

    c_tag = "KeyReference"
    c_namespace = NAMESPACE
    c_children = ReferenceType_.c_children.copy()
    c_attributes = ReferenceType_.c_attributes.copy()
    c_child_order = ReferenceType_.c_child_order[:]
    c_cardinality = ReferenceType_.c_cardinality.copy()


def key_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyReference, xml_string)


class ReferenceList(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:ReferenceList element"""

    c_tag = "ReferenceList"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}DataReference"] = ("data_reference", [DataReference])
    c_cardinality["data_reference"] = {"min": 0}
    c_children["{http://www.w3.org/2001/04/xmlenc#}KeyReference"] = ("key_reference", [KeyReference])
    c_cardinality["key_reference"] = {"min": 0}
    c_child_order.extend(["data_reference", "key_reference"])

    def __init__(
        self,
        data_reference=None,
        key_reference=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.data_reference = data_reference or []
        self.key_reference = key_reference or []


def reference_list_from_string(xml_string):
    return saml2.create_class_from_xml_string(ReferenceList, xml_string)


class EncryptionProperty(EncryptionPropertyType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionProperty element"""

    c_tag = "EncryptionProperty"
    c_namespace = NAMESPACE
    c_children = EncryptionPropertyType_.c_children.copy()
    c_attributes = EncryptionPropertyType_.c_attributes.copy()
    c_child_order = EncryptionPropertyType_.c_child_order[:]
    c_cardinality = EncryptionPropertyType_.c_cardinality.copy()


def encryption_property_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionProperty, xml_string)


class CipherReference(CipherReferenceType_):
    """The http://www.w3.org/2001/04/xmlenc#:CipherReference element"""

    c_tag = "CipherReference"
    c_namespace = NAMESPACE
    c_children = CipherReferenceType_.c_children.copy()
    c_attributes = CipherReferenceType_.c_attributes.copy()
    c_child_order = CipherReferenceType_.c_child_order[:]
    c_cardinality = CipherReferenceType_.c_cardinality.copy()


def cipher_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherReference, xml_string)


class EncryptionPropertiesType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionPropertiesType element"""

    c_tag = "EncryptionPropertiesType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}EncryptionProperty"] = ("encryption_property", [EncryptionProperty])
    c_cardinality["encryption_property"] = {"min": 1}
    c_attributes["Id"] = ("id", "ID", False)
    c_child_order.extend(["encryption_property"])

    def __init__(
        self,
        encryption_property=None,
        id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.encryption_property = encryption_property or []
        self.id = id


def encryption_properties_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionPropertiesType_, xml_string)


class CipherDataType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:CipherDataType element"""

    c_tag = "CipherDataType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}CipherValue"] = ("cipher_value", CipherValue)
    c_cardinality["cipher_value"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2001/04/xmlenc#}CipherReference"] = ("cipher_reference", CipherReference)
    c_cardinality["cipher_reference"] = {"min": 0, "max": 1}
    c_child_order.extend(["cipher_value", "cipher_reference"])

    def __init__(
        self,
        cipher_value=None,
        cipher_reference=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.cipher_value = cipher_value
        self.cipher_reference = cipher_reference


def cipher_data_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherDataType_, xml_string)


class EncryptionProperties(EncryptionPropertiesType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionProperties element"""

    c_tag = "EncryptionProperties"
    c_namespace = NAMESPACE
    c_children = EncryptionPropertiesType_.c_children.copy()
    c_attributes = EncryptionPropertiesType_.c_attributes.copy()
    c_child_order = EncryptionPropertiesType_.c_child_order[:]
    c_cardinality = EncryptionPropertiesType_.c_cardinality.copy()


def encryption_properties_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionProperties, xml_string)


class CipherData(CipherDataType_):
    """The http://www.w3.org/2001/04/xmlenc#:CipherData element"""

    c_tag = "CipherData"
    c_namespace = NAMESPACE
    c_children = CipherDataType_.c_children.copy()
    c_attributes = CipherDataType_.c_attributes.copy()
    c_child_order = CipherDataType_.c_child_order[:]
    c_cardinality = CipherDataType_.c_cardinality.copy()


def cipher_data_from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherData, xml_string)


class EncryptedType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedType element"""

    c_tag = "EncryptedType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}EncryptionMethod"] = ("encryption_method", EncryptionMethod)
    c_cardinality["encryption_method"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2000/09/xmldsig#}KeyInfo"] = ("key_info", ds.KeyInfo)
    c_cardinality["key_info"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2001/04/xmlenc#}CipherData"] = ("cipher_data", CipherData)
    c_children["{http://www.w3.org/2001/04/xmlenc#}EncryptionProperties"] = (
        "encryption_properties",
        EncryptionProperties,
    )
    c_cardinality["encryption_properties"] = {"min": 0, "max": 1}
    c_attributes["Id"] = ("id", "ID", False)
    c_attributes["Type"] = ("type", "anyURI", False)
    c_attributes["MimeType"] = ("mime_type", "string", False)
    c_attributes["Encoding"] = ("encoding", "anyURI", False)
    c_child_order.extend(["encryption_method", "key_info", "cipher_data", "encryption_properties"])

    def __init__(
        self,
        encryption_method=None,
        key_info=None,
        cipher_data=None,
        encryption_properties=None,
        id=None,
        type=None,
        mime_type=None,
        encoding=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.encryption_method = encryption_method
        self.key_info = key_info
        self.cipher_data = cipher_data
        self.encryption_properties = encryption_properties
        self.id = id
        self.type = type
        self.mime_type = mime_type
        self.encoding = encoding


class EncryptedDataType_(EncryptedType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedDataType element"""

    c_tag = "EncryptedDataType"
    c_namespace = NAMESPACE
    c_children = EncryptedType_.c_children.copy()
    c_attributes = EncryptedType_.c_attributes.copy()
    c_child_order = EncryptedType_.c_child_order[:]
    c_cardinality = EncryptedType_.c_cardinality.copy()


def encrypted_data_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedDataType_, xml_string)


class CarriedKeyName(SamlBase):

    c_tag = "CarriedKeyName"
    c_namespace = NAMESPACE
    c_value_type = {"base": "string"}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def carried_key_name_from_string(xml_string):
    return saml2.create_class_from_xml_string(CarriedKeyName, xml_string)


class EncryptedKeyType_(EncryptedType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedKeyType element"""

    c_tag = "EncryptedKeyType"
    c_namespace = NAMESPACE
    c_children = EncryptedType_.c_children.copy()
    c_attributes = EncryptedType_.c_attributes.copy()
    c_child_order = EncryptedType_.c_child_order[:]
    c_cardinality = EncryptedType_.c_cardinality.copy()
    c_children["{http://www.w3.org/2001/04/xmlenc#}ReferenceList"] = ("reference_list", ReferenceList)
    c_cardinality["reference_list"] = {"min": 0, "max": 1}
    c_children["{http://www.w3.org/2001/04/xmlenc#}CarriedKeyName"] = ("carried_key_name", CarriedKeyName)
    c_cardinality["carried_key_name"] = {"min": 0, "max": 1}
    c_attributes["Recipient"] = ("recipient", "string", False)
    c_child_order.extend(["reference_list", "carried_key_name"])

    def __init__(
        self,
        reference_list=None,
        carried_key_name=None,
        recipient=None,
        encryption_method=None,
        key_info=None,
        cipher_data=None,
        encryption_properties=None,
        id=None,
        type=None,
        mime_type=None,
        encoding=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        EncryptedType_.__init__(
            self,
            encryption_method=encryption_method,
            key_info=key_info,
            cipher_data=cipher_data,
            encryption_properties=encryption_properties,
            id=id,
            type=type,
            mime_type=mime_type,
            encoding=encoding,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.reference_list = reference_list
        self.carried_key_name = carried_key_name
        self.recipient = recipient


def encrypted_key_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedKeyType_, xml_string)


class EncryptedData(EncryptedDataType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedData element"""

    c_tag = "EncryptedData"
    c_namespace = NAMESPACE
    c_children = EncryptedDataType_.c_children.copy()
    c_attributes = EncryptedDataType_.c_attributes.copy()
    c_child_order = EncryptedDataType_.c_child_order[:]
    c_cardinality = EncryptedDataType_.c_cardinality.copy()


def encrypted_data_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedData, xml_string)


class EncryptedKey(EncryptedKeyType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedKey element"""

    c_tag = "EncryptedKey"
    c_namespace = NAMESPACE
    c_children = EncryptedKeyType_.c_children.copy()
    c_attributes = EncryptedKeyType_.c_attributes.copy()
    c_child_order = EncryptedKeyType_.c_child_order[:]
    c_cardinality = EncryptedKeyType_.c_cardinality.copy()


def encrypted_key_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedKey, xml_string)


ds.KeyInfo.c_children["{http://www.w3.org/2000/09/xmlenc#}EncryptedKey"] = ("encrypted_key", EncryptedKey)


ELEMENT_FROM_STRING = {
    EncryptionMethodType_.c_tag: encryption_method_type__from_string,
    KeySizeType_.c_tag: key_size_type__from_string,
    CipherData.c_tag: cipher_data_from_string,
    CipherDataType_.c_tag: cipher_data_type__from_string,
    CipherReference.c_tag: cipher_reference_from_string,
    CipherReferenceType_.c_tag: cipher_reference_type__from_string,
    TransformsType_.c_tag: transforms_type__from_string,
    EncryptedData.c_tag: encrypted_data_from_string,
    EncryptedDataType_.c_tag: encrypted_data_type__from_string,
    EncryptedKey.c_tag: encrypted_key_from_string,
    EncryptedKeyType_.c_tag: encrypted_key_type__from_string,
    AgreementMethod.c_tag: agreement_method_from_string,
    AgreementMethodType_.c_tag: agreement_method_type__from_string,
    ReferenceList.c_tag: reference_list_from_string,
    ReferenceType_.c_tag: reference_type__from_string,
    EncryptionProperties.c_tag: encryption_properties_from_string,
    EncryptionPropertiesType_.c_tag: encryption_properties_type__from_string,
    EncryptionProperty.c_tag: encryption_property_from_string,
    EncryptionPropertyType_.c_tag: encryption_property_type__from_string,
    CipherValue.c_tag: cipher_value_from_string,
    KA_Nonce.c_tag: k_a__nonce_from_string,
    OriginatorKeyInfo.c_tag: originator_key_info_from_string,
    RecipientKeyInfo.c_tag: recipient_key_info_from_string,
    KeySize.c_tag: key_size_from_string,
    OAEPparams.c_tag: oae_pparams_from_string,
    Transforms.c_tag: transforms_from_string,
    EncryptionMethod.c_tag: encryption_method_from_string,
    DataReference.c_tag: data_reference_from_string,
    KeyReference.c_tag: key_reference_from_string,
    CarriedKeyName.c_tag: carried_key_name_from_string,
}

ELEMENT_BY_TAG = {
    "EncryptionMethodType": EncryptionMethodType_,
    "KeySizeType": KeySizeType_,
    "CipherData": CipherData,
    "CipherDataType": CipherDataType_,
    "CipherReference": CipherReference,
    "CipherReferenceType": CipherReferenceType_,
    "TransformsType": TransformsType_,
    "EncryptedData": EncryptedData,
    "EncryptedDataType": EncryptedDataType_,
    "EncryptedKey": EncryptedKey,
    "EncryptedKeyType": EncryptedKeyType_,
    "AgreementMethod": AgreementMethod,
    "AgreementMethodType": AgreementMethodType_,
    "ReferenceList": ReferenceList,
    "ReferenceType": ReferenceType_,
    "EncryptionProperties": EncryptionProperties,
    "EncryptionPropertiesType": EncryptionPropertiesType_,
    "EncryptionProperty": EncryptionProperty,
    "EncryptionPropertyType": EncryptionPropertyType_,
    "CipherValue": CipherValue,
    "KA_Nonce": KA_Nonce,
    "OriginatorKeyInfo": OriginatorKeyInfo,
    "RecipientKeyInfo": RecipientKeyInfo,
    "KeySize": KeySize,
    "OAEPparams": OAEPparams,
    "Transforms": Transforms,
    "EncryptionMethod": EncryptionMethod,
    "DataReference": DataReference,
    "KeyReference": KeyReference,
    "CarriedKeyName": CarriedKeyName,
    "EncryptedType": EncryptedType_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:basic",
    "fro": {
        "urn:mace:dir:attribute-def:aRecord": "aRecord",
        "urn:mace:dir:attribute-def:aliasedEntryName": "aliasedEntryName",
        "urn:mace:dir:attribute-def:aliasedObjectName": "aliasedObjectName",
        "urn:mace:dir:attribute-def:associatedDomain": "associatedDomain",
        "urn:mace:dir:attribute-def:associatedName": "associatedName",
        "urn:mace:dir:attribute-def:audio": "audio",
        "urn:mace:dir:attribute-def:authorityRevocationList": "authorityRevocationList",
        "urn:mace:dir:attribute-def:buildingName": "buildingName",
        "urn:mace:dir:attribute-def:businessCategory": "businessCategory",
        "urn:mace:dir:attribute-def:c": "c",
        "urn:mace:dir:attribute-def:cACertificate": "cACertificate",
        "urn:mace:dir:attribute-def:cNAMERecord": "cNAMERecord",
        "urn:mace:dir:attribute-def:carLicense": "carLicense",
        "urn:mace:dir:attribute-def:certificateRevocationList": "certificateRevocationList",
        "urn:mace:dir:attribute-def:cn": "cn",
        "urn:mace:dir:attribute-def:co": "co",
        "urn:mace:dir:attribute-def:commonName": "commonName",
        "urn:mace:dir:attribute-def:countryName": "countryName",
        "urn:mace:dir:attribute-def:crossCertificatePair": "crossCertificatePair",
        "urn:mace:dir:attribute-def:dITRedirect": "dITRedirect",
        "urn:mace:dir:attribute-def:dSAQuality": "dSAQuality",
        "urn:mace:dir:attribute-def:dc": "dc",
        "urn:mace:dir:attribute-def:deltaRevocationList": "deltaRevocationList",
        "urn:mace:dir:attribute-def:departmentNumber": "departmentNumber",
        "urn:mace:dir:attribute-def:description": "description",
        "urn:mace:dir:attribute-def:destinationIndicator": "destinationIndicator",
        "urn:mace:dir:attribute-def:displayName": "displayName",
        "urn:mace:dir:attribute-def:distinguishedName": "distinguishedName",
        "urn:mace:dir:attribute-def:dmdName": "dmdName",
        "urn:mace:dir:attribute-def:dnQualifier": "dnQualifier",
        "urn:mace:dir:attribute-def:documentAuthor": "documentAuthor",
        "urn:mace:dir:attribute-def:documentIdentifier": "documentIdentifier",
        "urn:mace:dir:attribute-def:documentLocation": "documentLocation",
        "urn:mace:dir:attribute-def:documentPublisher": "documentPublisher",
        "urn:mace:dir:attribute-def:documentTitle": "documentTitle",
        "urn:mace:dir:attribute-def:documentVersion": "documentVersion",
        "urn:mace:dir:attribute-def:domainComponent": "domainComponent",
        "urn:mace:dir:attribute-def:drink": "drink",
        "urn:mace:dir:attribute-def:eduOrgHomePageURI": "eduOrgHomePageURI",
        "urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI": "eduOrgIdentityAuthNPolicyURI",
        "urn:mace:dir:attribute-def:eduOrgLegalName": "eduOrgLegalName",
        "urn:mace:dir:attribute-def:eduOrgSuperiorURI": "eduOrgSuperiorURI",
        "urn:mace:dir:attribute-def:eduOrgWhitePagesURI": "eduOrgWhitePagesURI",
        "urn:mace:dir:attribute-def:eduPersonAffiliation": "eduPersonAffiliation",
        "urn:mace:dir:attribute-def:eduPersonEntitlement": "eduPersonEntitlement",
        "urn:mace:dir:attribute-def:eduPersonNickname": "eduPersonNickname",
        "urn:mace:dir:attribute-def:eduPersonOrgDN": "eduPersonOrgDN",
        "urn:mace:dir:attribute-def:eduPersonOrgUnitDN": "eduPersonOrgUnitDN",
        "urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation": "eduPersonPrimaryAffiliation",
        "urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN": "eduPersonPrimaryOrgUnitDN",
        "urn:mace:dir:attribute-def:eduPersonPrincipalName": "eduPersonPrincipalName",
        "urn:mace:dir:attribute-def:eduPersonScopedAffiliation": "eduPersonScopedAffiliation",
        "urn:mace:dir:attribute-def:eduPersonTargetedID": "eduPersonTargetedID",
        "urn:mace:dir:attribute-def:email": "email",
        "urn:mace:dir:attribute-def:emailAddress": "emailAddress",
        "urn:mace:dir:attribute-def:employeeNumber": "employeeNumber",
        "urn:mace:dir:attribute-def:employeeType": "employeeType",
        "urn:mace:dir:attribute-def:enhancedSearchGuide": "enhancedSearchGuide",
        "urn:mace:dir:attribute-def:facsimileTelephoneNumber": "facsimileTelephoneNumber",
        "urn:mace:dir:attribute-def:favouriteDrink": "favouriteDrink",
        "urn:mace:dir:attribute-def:fax": "fax",
        "urn:mace:dir:attribute-def:federationFeideSchemaVersion": "federationFeideSchemaVersion",
        "urn:mace:dir:attribute-def:friendlyCountryName": "friendlyCountryName",
        "urn:mace:dir:attribute-def:generationQualifier": "generationQualifier",
        "urn:mace:dir:attribute-def:givenName": "givenName",
        "urn:mace:dir:attribute-def:gn": "gn",
        "urn:mace:dir:attribute-def:homePhone": "homePhone",
        "urn:mace:dir:attribute-def:homePostalAddress": "homePostalAddress",
        "urn:mace:dir:attribute-def:homeTelephoneNumber": "homeTelephoneNumber",
        "urn:mace:dir:attribute-def:host": "host",
        "urn:mace:dir:attribute-def:houseIdentifier": "houseIdentifier",
        "urn:mace:dir:attribute-def:info": "info",
        "urn:mace:dir:attribute-def:initials": "initials",
        "urn:mace:dir:attribute-def:internationaliSDNNumber": "internationaliSDNNumber",
        "urn:mace:dir:attribute-def:janetMailbox": "janetMailbox",
        "urn:mace:dir:attribute-def:jpegPhoto": "jpegPhoto",
        "urn:mace:dir:attribute-def:knowledgeInformation": "knowledgeInformation",
        "urn:mace:dir:attribute-def:l": "l",
        "urn:mace:dir:attribute-def:labeledURI": "labeledURI",
        "urn:mace:dir:attribute-def:localityName": "localityName",
        "urn:mace:dir:attribute-def:mDRecord": "mDRecord",
        "urn:mace:dir:attribute-def:mXRecord": "mXRecord",
        "urn:mace:dir:attribute-def:mail": "mail",
        "urn:mace:dir:attribute-def:mailPreferenceOption": "mailPreferenceOption",
        "urn:mace:dir:attribute-def:manager": "manager",
        "urn:mace:dir:attribute-def:member": "member",
        "urn:mace:dir:attribute-def:mobile": "mobile",
        "urn:mace:dir:attribute-def:mobileTelephoneNumber": "mobileTelephoneNumber",
        "urn:mace:dir:attribute-def:nSRecord": "nSRecord",
        "urn:mace:dir:attribute-def:name": "name",
        "urn:mace:dir:attribute-def:norEduOrgAcronym": "norEduOrgAcronym",
        "urn:mace:dir:attribute-def:norEduOrgNIN": "norEduOrgNIN",
        "urn:mace:dir:attribute-def:norEduOrgSchemaVersion": "norEduOrgSchemaVersion",
        "urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier": "norEduOrgUniqueIdentifier",
        "urn:mace:dir:attribute-def:norEduOrgUniqueNumber": "norEduOrgUniqueNumber",
        "urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier": "norEduOrgUnitUniqueIdentifier",
        "urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber": "norEduOrgUnitUniqueNumber",
        "urn:mace:dir:attribute-def:norEduPersonBirthDate": "norEduPersonBirthDate",
        "urn:mace:dir:attribute-def:norEduPersonLIN": "norEduPersonLIN",
        "urn:mace:dir:attribute-def:norEduPersonNIN": "norEduPersonNIN",
        "urn:mace:dir:attribute-def:o": "o",
        "urn:mace:dir:attribute-def:objectClass": "objectClass",
        "urn:mace:dir:attribute-def:organizationName": "organizationName",
        "urn:mace:dir:attribute-def:organizationalStatus": "organizationalStatus",
        "urn:mace:dir:attribute-def:organizationalUnitName": "organizationalUnitName",
        "urn:mace:dir:attribute-def:otherMailbox": "otherMailbox",
        "urn:mace:dir:attribute-def:ou": "ou",
        "urn:mace:dir:attribute-def:owner": "owner",
        "urn:mace:dir:attribute-def:pager": "pager",
        "urn:mace:dir:attribute-def:pagerTelephoneNumber": "pagerTelephoneNumber",
        "urn:mace:dir:attribute-def:personalSignature": "personalSignature",
        "urn:mace:dir:attribute-def:personalTitle": "personalTitle",
        "urn:mace:dir:attribute-def:photo": "photo",
        "urn:mace:dir:attribute-def:physicalDeliveryOfficeName": "physicalDeliveryOfficeName",
        "urn:mace:dir:attribute-def:pkcs9email": "pkcs9email",
        "urn:mace:dir:attribute-def:postOfficeBox": "postOfficeBox",
        "urn:mace:dir:attribute-def:postalAddress": "postalAddress",
        "urn:mace:dir:attribute-def:postalCode": "postalCode",
        "urn:mace:dir:attribute-def:preferredDeliveryMethod": "preferredDeliveryMethod",
        "urn:mace:dir:attribute-def:preferredLanguage": "preferredLanguage",
        "urn:mace:dir:attribute-def:presentationAddress": "presentationAddress",
        "urn:mace:dir:attribute-def:protocolInformation": "protocolInformation",
        "urn:mace:dir:attribute-def:pseudonym": "pseudonym",
        "urn:mace:dir:attribute-def:registeredAddress": "registeredAddress",
        "urn:mace:dir:attribute-def:rfc822Mailbox": "rfc822Mailbox",
        "urn:mace:dir:attribute-def:roleOccupant": "roleOccupant",
        "urn:mace:dir:attribute-def:roomNumber": "roomNumber",
        "urn:mace:dir:attribute-def:sOARecord": "sOARecord",
        "urn:mace:dir:attribute-def:searchGuide": "searchGuide",
        "urn:mace:dir:attribute-def:secretary": "secretary",
        "urn:mace:dir:attribute-def:seeAlso": "seeAlso",
        "urn:mace:dir:attribute-def:serialNumber": "serialNumber",
        "urn:mace:dir:attribute-def:singleLevelQuality": "singleLevelQuality",
        "urn:mace:dir:attribute-def:sn": "sn",
        "urn:mace:dir:attribute-def:st": "st",
        "urn:mace:dir:attribute-def:stateOrProvinceName": "stateOrProvinceName",
        "urn:mace:dir:attribute-def:street": "street",
        "urn:mace:dir:attribute-def:streetAddress": "streetAddress",
        "urn:mace:dir:attribute-def:subtreeMaximumQuality": "subtreeMaximumQuality",
        "urn:mace:dir:attribute-def:subtreeMinimumQuality": "subtreeMinimumQuality",
        "urn:mace:dir:attribute-def:supportedAlgorithms": "supportedAlgorithms",
        "urn:mace:dir:attribute-def:supportedApplicationContext": "supportedApplicationContext",
        "urn:mace:dir:attribute-def:surname": "surname",
        "urn:mace:dir:attribute-def:telephoneNumber": "telephoneNumber",
        "urn:mace:dir:attribute-def:teletexTerminalIdentifier": "teletexTerminalIdentifier",
        "urn:mace:dir:attribute-def:telexNumber": "telexNumber",
        "urn:mace:dir:attribute-def:textEncodedORAddress": "textEncodedORAddress",
        "urn:mace:dir:attribute-def:title": "title",
        "urn:mace:dir:attribute-def:uid": "uid",
        "urn:mace:dir:attribute-def:uniqueIdentifier": "uniqueIdentifier",
        "urn:mace:dir:attribute-def:uniqueMember": "uniqueMember",
        "urn:mace:dir:attribute-def:userCertificate": "userCertificate",
        "urn:mace:dir:attribute-def:userClass": "userClass",
        "urn:mace:dir:attribute-def:userPKCS12": "userPKCS12",
        "urn:mace:dir:attribute-def:userPassword": "userPassword",
        "urn:mace:dir:attribute-def:userSMIMECertificate": "userSMIMECertificate",
        "urn:mace:dir:attribute-def:userid": "userid",
        "urn:mace:dir:attribute-def:x121Address": "x121Address",
        "urn:mace:dir:attribute-def:x500UniqueIdentifier": "x500UniqueIdentifier",
    },
    "to": {
        "aRecord": "urn:mace:dir:attribute-def:aRecord",
        "aliasedEntryName": "urn:mace:dir:attribute-def:aliasedEntryName",
        "aliasedObjectName": "urn:mace:dir:attribute-def:aliasedObjectName",
        "associatedDomain": "urn:mace:dir:attribute-def:associatedDomain",
        "associatedName": "urn:mace:dir:attribute-def:associatedName",
        "audio": "urn:mace:dir:attribute-def:audio",
        "authorityRevocationList": "urn:mace:dir:attribute-def:authorityRevocationList",
        "buildingName": "urn:mace:dir:attribute-def:buildingName",
        "businessCategory": "urn:mace:dir:attribute-def:businessCategory",
        "c": "urn:mace:dir:attribute-def:c",
        "cACertificate": "urn:mace:dir:attribute-def:cACertificate",
        "cNAMERecord": "urn:mace:dir:attribute-def:cNAMERecord",
        "carLicense": "urn:mace:dir:attribute-def:carLicense",
        "certificateRevocationList": "urn:mace:dir:attribute-def:certificateRevocationList",
        "cn": "urn:mace:dir:attribute-def:cn",
        "co": "urn:mace:dir:attribute-def:co",
        "commonName": "urn:mace:dir:attribute-def:commonName",
        "countryName": "urn:mace:dir:attribute-def:countryName",
        "crossCertificatePair": "urn:mace:dir:attribute-def:crossCertificatePair",
        "dITRedirect": "urn:mace:dir:attribute-def:dITRedirect",
        "dSAQuality": "urn:mace:dir:attribute-def:dSAQuality",
        "dc": "urn:mace:dir:attribute-def:dc",
        "deltaRevocationList": "urn:mace:dir:attribute-def:deltaRevocationList",
        "departmentNumber": "urn:mace:dir:attribute-def:departmentNumber",
        "description": "urn:mace:dir:attribute-def:description",
        "destinationIndicator": "urn:mace:dir:attribute-def:destinationIndicator",
        "displayName": "urn:mace:dir:attribute-def:displayName",
        "distinguishedName": "urn:mace:dir:attribute-def:distinguishedName",
        "dmdName": "urn:mace:dir:attribute-def:dmdName",
        "dnQualifier": "urn:mace:dir:attribute-def:dnQualifier",
        "documentAuthor": "urn:mace:dir:attribute-def:documentAuthor",
        "documentIdentifier": "urn:mace:dir:attribute-def:documentIdentifier",
        "documentLocation": "urn:mace:dir:attribute-def:documentLocation",
        "documentPublisher": "urn:mace:dir:attribute-def:documentPublisher",
        "documentTitle": "urn:mace:dir:attribute-def:documentTitle",
        "documentVersion": "urn:mace:dir:attribute-def:documentVersion",
        "domainComponent": "urn:mace:dir:attribute-def:domainComponent",
        "drink": "urn:mace:dir:attribute-def:drink",
        "eduOrgHomePageURI": "urn:mace:dir:attribute-def:eduOrgHomePageURI",
        "eduOrgIdentityAuthNPolicyURI": "urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI",
        "eduOrgLegalName": "urn:mace:dir:attribute-def:eduOrgLegalName",
        "eduOrgSuperiorURI": "urn:mace:dir:attribute-def:eduOrgSuperiorURI",
        "eduOrgWhitePagesURI": "urn:mace:dir:attribute-def:eduOrgWhitePagesURI",
        "eduPersonAffiliation": "urn:mace:dir:attribute-def:eduPersonAffiliation",
        "eduPersonEntitlement": "urn:mace:dir:attribute-def:eduPersonEntitlement",
        "eduPersonNickname": "urn:mace:dir:attribute-def:eduPersonNickname",
        "eduPersonOrgDN": "urn:mace:dir:attribute-def:eduPersonOrgDN",
        "eduPersonOrgUnitDN": "urn:mace:dir:attribute-def:eduPersonOrgUnitDN",
        "eduPersonPrimaryAffiliation": "urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation",
        "eduPersonPrimaryOrgUnitDN": "urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN",
        "eduPersonPrincipalName": "urn:mace:dir:attribute-def:eduPersonPrincipalName",
        "eduPersonScopedAffiliation": "urn:mace:dir:attribute-def:eduPersonScopedAffiliation",
        "eduPersonTargetedID": "urn:mace:dir:attribute-def:eduPersonTargetedID",
        "email": "urn:mace:dir:attribute-def:email",
        "emailAddress": "urn:mace:dir:attribute-def:emailAddress",
        "employeeNumber": "urn:mace:dir:attribute-def:employeeNumber",
        "employeeType": "urn:mace:dir:attribute-def:employeeType",
        "enhancedSearchGuide": "urn:mace:dir:attribute-def:enhancedSearchGuide",
        "facsimileTelephoneNumber": "urn:mace:dir:attribute-def:facsimileTelephoneNumber",
        "favouriteDrink": "urn:mace:dir:attribute-def:favouriteDrink",
        "fax": "urn:mace:dir:attribute-def:fax",
        "federationFeideSchemaVersion": "urn:mace:dir:attribute-def:federationFeideSchemaVersion",
        "friendlyCountryName": "urn:mace:dir:attribute-def:friendlyCountryName",
        "generationQualifier": "urn:mace:dir:attribute-def:generationQualifier",
        "givenName": "urn:mace:dir:attribute-def:givenName",
        "gn": "urn:mace:dir:attribute-def:gn",
        "homePhone": "urn:mace:dir:attribute-def:homePhone",
        "homePostalAddress": "urn:mace:dir:attribute-def:homePostalAddress",
        "homeTelephoneNumber": "urn:mace:dir:attribute-def:homeTelephoneNumber",
        "host": "urn:mace:dir:attribute-def:host",
        "houseIdentifier": "urn:mace:dir:attribute-def:houseIdentifier",
        "info": "urn:mace:dir:attribute-def:info",
        "initials": "urn:mace:dir:attribute-def:initials",
        "internationaliSDNNumber": "urn:mace:dir:attribute-def:internationaliSDNNumber",
        "janetMailbox": "urn:mace:dir:attribute-def:janetMailbox",
        "jpegPhoto": "urn:mace:dir:attribute-def:jpegPhoto",
        "knowledgeInformation": "urn:mace:dir:attribute-def:knowledgeInformation",
        "l": "urn:mace:dir:attribute-def:l",
        "labeledURI": "urn:mace:dir:attribute-def:labeledURI",
        "localityName": "urn:mace:dir:attribute-def:localityName",
        "mDRecord": "urn:mace:dir:attribute-def:mDRecord",
        "mXRecord": "urn:mace:dir:attribute-def:mXRecord",
        "mail": "urn:mace:dir:attribute-def:mail",
        "mailPreferenceOption": "urn:mace:dir:attribute-def:mailPreferenceOption",
        "manager": "urn:mace:dir:attribute-def:manager",
        "member": "urn:mace:dir:attribute-def:member",
        "mobile": "urn:mace:dir:attribute-def:mobile",
        "mobileTelephoneNumber": "urn:mace:dir:attribute-def:mobileTelephoneNumber",
        "nSRecord": "urn:mace:dir:attribute-def:nSRecord",
        "name": "urn:mace:dir:attribute-def:name",
        "norEduOrgAcronym": "urn:mace:dir:attribute-def:norEduOrgAcronym",
        "norEduOrgNIN": "urn:mace:dir:attribute-def:norEduOrgNIN",
        "norEduOrgSchemaVersion": "urn:mace:dir:attribute-def:norEduOrgSchemaVersion",
        "norEduOrgUniqueIdentifier": "urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier",
        "norEduOrgUniqueNumber": "urn:mace:dir:attribute-def:norEduOrgUniqueNumber",
        "norEduOrgUnitUniqueIdentifier": "urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier",
        "norEduOrgUnitUniqueNumber": "urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber",
        "norEduPersonBirthDate": "urn:mace:dir:attribute-def:norEduPersonBirthDate",
        "norEduPersonLIN": "urn:mace:dir:attribute-def:norEduPersonLIN",
        "norEduPersonNIN": "urn:mace:dir:attribute-def:norEduPersonNIN",
        "o": "urn:mace:dir:attribute-def:o",
        "objectClass": "urn:mace:dir:attribute-def:objectClass",
        "organizationName": "urn:mace:dir:attribute-def:organizationName",
        "organizationalStatus": "urn:mace:dir:attribute-def:organizationalStatus",
        "organizationalUnitName": "urn:mace:dir:attribute-def:organizationalUnitName",
        "otherMailbox": "urn:mace:dir:attribute-def:otherMailbox",
        "ou": "urn:mace:dir:attribute-def:ou",
        "owner": "urn:mace:dir:attribute-def:owner",
        "pager": "urn:mace:dir:attribute-def:pager",
        "pagerTelephoneNumber": "urn:mace:dir:attribute-def:pagerTelephoneNumber",
        "personalSignature": "urn:mace:dir:attribute-def:personalSignature",
        "personalTitle": "urn:mace:dir:attribute-def:personalTitle",
        "photo": "urn:mace:dir:attribute-def:photo",
        "physicalDeliveryOfficeName": "urn:mace:dir:attribute-def:physicalDeliveryOfficeName",
        "pkcs9email": "urn:mace:dir:attribute-def:pkcs9email",
        "postOfficeBox": "urn:mace:dir:attribute-def:postOfficeBox",
        "postalAddress": "urn:mace:dir:attribute-def:postalAddress",
        "postalCode": "urn:mace:dir:attribute-def:postalCode",
        "preferredDeliveryMethod": "urn:mace:dir:attribute-def:preferredDeliveryMethod",
        "preferredLanguage": "urn:mace:dir:attribute-def:preferredLanguage",
        "presentationAddress": "urn:mace:dir:attribute-def:presentationAddress",
        "protocolInformation": "urn:mace:dir:attribute-def:protocolInformation",
        "pseudonym": "urn:mace:dir:attribute-def:pseudonym",
        "registeredAddress": "urn:mace:dir:attribute-def:registeredAddress",
        "rfc822Mailbox": "urn:mace:dir:attribute-def:rfc822Mailbox",
        "roleOccupant": "urn:mace:dir:attribute-def:roleOccupant",
        "roomNumber": "urn:mace:dir:attribute-def:roomNumber",
        "sOARecord": "urn:mace:dir:attribute-def:sOARecord",
        "searchGuide": "urn:mace:dir:attribute-def:searchGuide",
        "secretary": "urn:mace:dir:attribute-def:secretary",
        "seeAlso": "urn:mace:dir:attribute-def:seeAlso",
        "serialNumber": "urn:mace:dir:attribute-def:serialNumber",
        "singleLevelQuality": "urn:mace:dir:attribute-def:singleLevelQuality",
        "sn": "urn:mace:dir:attribute-def:sn",
        "st": "urn:mace:dir:attribute-def:st",
        "stateOrProvinceName": "urn:mace:dir:attribute-def:stateOrProvinceName",
        "street": "urn:mace:dir:attribute-def:street",
        "streetAddress": "urn:mace:dir:attribute-def:streetAddress",
        "subtreeMaximumQuality": "urn:mace:dir:attribute-def:subtreeMaximumQuality",
        "subtreeMinimumQuality": "urn:mace:dir:attribute-def:subtreeMinimumQuality",
        "supportedAlgorithms": "urn:mace:dir:attribute-def:supportedAlgorithms",
        "supportedApplicationContext": "urn:mace:dir:attribute-def:supportedApplicationContext",
        "surname": "urn:mace:dir:attribute-def:surname",
        "telephoneNumber": "urn:mace:dir:attribute-def:telephoneNumber",
        "teletexTerminalIdentifier": "urn:mace:dir:attribute-def:teletexTerminalIdentifier",
        "telexNumber": "urn:mace:dir:attribute-def:telexNumber",
        "textEncodedORAddress": "urn:mace:dir:attribute-def:textEncodedORAddress",
        "title": "urn:mace:dir:attribute-def:title",
        "uid": "urn:mace:dir:attribute-def:uid",
        "uniqueIdentifier": "urn:mace:dir:attribute-def:uniqueIdentifier",
        "uniqueMember": "urn:mace:dir:attribute-def:uniqueMember",
        "userCertificate": "urn:mace:dir:attribute-def:userCertificate",
        "userClass": "urn:mace:dir:attribute-def:userClass",
        "userPKCS12": "urn:mace:dir:attribute-def:userPKCS12",
        "userPassword": "urn:mace:dir:attribute-def:userPassword",
        "userSMIMECertificate": "urn:mace:dir:attribute-def:userSMIMECertificate",
        "userid": "urn:mace:dir:attribute-def:userid",
        "x121Address": "urn:mace:dir:attribute-def:x121Address",
        "x500UniqueIdentifier": "urn:mace:dir:attribute-def:x500UniqueIdentifier",
    },
}



#!/usr/bin/env python
from saml2.saml import NAME_FORMAT_URI


__author__ = "rolandh"

import json

from saml2 import xmldsig as ds
from saml2.saml import NAME_FORMAT_URI


BASE = "http://localhost:8088"

metadata = open("./idp/idp.xml").read()

info = {
    "entity_id": f"{BASE}/idp.xml",
    "interaction": [
        {
            "matches": {"url": f"{BASE}/sso/redirect", "title": "IDP test login"},
            "page-type": "login",
            "control": {"type": "form", "set": {"login": "roland", "password": "dianakra"}},
        },
        {
            "matches": {"url": f"{BASE}/sso/post", "title": "IDP test login"},
            "page-type": "login",
            "control": {"type": "form", "set": {"login": "roland", "password": "dianakra"}},
        },
        {
            "matches": {"url": f"{BASE}/sso/redirect", "title": "SAML 2.0 POST"},
            "page-type": "other",
            "control": {
                "index": 0,
                "type": "form",
            },
        },
        {
            "matches": {"url": f"{BASE}/sso/post", "title": "SAML 2.0 POST"},
            "page-type": "other",
            "control": {"index": 0, "type": "form", "set": {}},
        },
        {
            "matches": {"url": f"{BASE}/slo/post", "title": "SAML 2.0 POST"},
            "page-type": "other",
            "control": {"index": 0, "type": "form", "set": {}},
        },
    ],
    # metadata source for the test target's EntityDescriptor:
    "metadata": metadata,
    "constraints": {
        # test if attribute name format matches the given value. Absence of this
        # option or the value NAME_FORMAT_UNSPECIFIED will match any format
        # "name_format": NAME_FORMAT_BASIC,
        # "name_format": NAME_FORMAT_UNSPECIFIED,
        "name_format": NAME_FORMAT_URI,
        # allowed for assertion & response:
        "signature_algorithm": [
            # ds.SIG_RSA_SHA1,  # you may need this for legacy deployments
            ds.SIG_RSA_SHA224,
            ds.SIG_RSA_SHA256,
            ds.SIG_RSA_SHA384,
            ds.SIG_RSA_SHA512,
        ],
        "digest_algorithm": [
            # ds.DIGEST_SHA1,   # you may need this for legacy deployments
            ds.DIGEST_SHA224,
            ds.DIGEST_SHA256,
            ds.DIGEST_SHA384,
            ds.DIGEST_SHA512,
            ds.DIGEST_RIPEMD160,
        ],
    },
}

print(json.dumps(info))



# See http://technet.microsoft.com/en-us/library/ee913589(v=ws.10).aspx
# for information regarding the default claim types supported by
# Microsoft ADFS v2.0.

MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:unspecified",
    "fro": {
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress": "emailAddress",
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname": "givenName",
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": "name",
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/upn": "upn",
        "http://schemas.xmlsoap.org/claims/commonname": "commonName",
        "http://schemas.xmlsoap.org/claims/group": "group",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/role": "role",
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname": "surname",
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/privatepersonalidentifier": "privatePersonalId",
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/nameidentifier": "nameId",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/authenticationmethod": "authenticationMethod",
        "http://schemas.xmlsoap.com/ws/2005/05/identity/claims/denyonlysid": "denyOnlySid",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/denyonlyprimarysid": "denyOnlyPrimarySid",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/denyonlyprimarygroupsid": "denyOnlyPrimaryGroupSid",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/groupsid": "groupSid",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/primarygroupsid": "primaryGroupSid",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/primarysid": "primarySid",
        "http://schemas.microsoft.com/ws/2008/06/identity/claims/windowsaccountname": "windowsAccountName",
    },
    "to": {
        "emailAddress": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress",
        "givenName": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname",
        "name": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name",
        "upn": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/upn",
        "commonName": "http://schemas.xmlsoap.org/claims/commonname",
        "group": "http://schemas.xmlsoap.org/claims/group",
        "role": "http://schemas.microsoft.com/ws/2008/06/identity/claims/role",
        "surname": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname",
        "privatePersonalId": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/privatepersonalidentifier",
        "nameId": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/nameidentifier",
        "authenticationMethod": "http://schemas.microsoft.com/ws/2008/06/identity/claims/authenticationmethod",
        "denyOnlySid": "http://schemas.xmlsoap.com/ws/2005/05/identity/claims/denyonlysid",
        "denyOnlyPrimarySid": "http://schemas.microsoft.com/ws/2008/06/identity/claims/denyonlyprimarysid",
        "denyOnlyPrimaryGroupSid": "http://schemas.microsoft.com/ws/2008/06/identity/claims/denyonlyprimarygroupsid",
        "groupSid": "http://schemas.microsoft.com/ws/2008/06/identity/claims/groupsid",
        "primaryGroupSid": "http://schemas.microsoft.com/ws/2008/06/identity/claims/primarygroupsid",
        "primarySid": "http://schemas.microsoft.com/ws/2008/06/identity/claims/primarysid",
        "windowsAccountName": "http://schemas.microsoft.com/ws/2008/06/identity/claims/windowsaccountname",
    },
}
